<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 *
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version.
 * Enqueues the admin-specific stylesheet and JavaScript.
 * Defines core functionality of the plugin and acts as controller.
 *
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The post type to store data about form submissions.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $post_type    The post type that will be queried and displayed.
	 */
	private $post_type;

	/**
	 * The post status to indicate a spam message.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $spam_status   The post status used to indicate spam.
	 */
	private $spam_status;
	
	/**
	 * Which tracking status exist and must be displayed
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $tracking_statuses   The statuses that will be displayed (key -> value).
	 */
	private $tracking_statuses;

	/**
	 * How many posts (= inbound messages) to show initially and how many to add when loading more
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      integer    $post_per_page   The value that determines the posts per page
	 */
	private $posts_per_page;

	/**
	 * Array to instruct WP_Query how to sort posts
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $sorting  The array to include as sorting parameter for WP_Query
	 */
	private $sorting;
	

	/**
	 * Array that maps color choice to class names
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $color_classes  The array that maps numbers to class names for colors 
	 */
	private $color_classes;
	

	/**
	 * Instance of plugin class Inbound_Organizer_Plugins
	 */
	private $plugins;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since 1.0.0
	 * @param string    $plugin_name      The name of this plugin.
	 * @param string    $version    	  The version of this plugin.
	 * @param Inbound_Organizer_Plugins	  The instance of the plugins class
	 */
	public function __construct( $plugin_name, $version, $plugins ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->post_type = 'inborg_item';
		$this->spam_status = 'inborg_spam_status';
		$this->plugins = $plugins;
		$this->tracking_statuses = array();
		
		$column_options = get_option( 'inbound_organizer_columns' );
		if ( $column_options ) {
			$columns = $column_options['number_of_columns'];
			for ( $column = 1; $column <= $columns; $column++ ) {
				$column_name = array_key_exists( 'column_name_'.$column, $column_options ) ?
					$column_options['column_name_'.$column] : '';
				$this->tracking_statuses[ $column ] = $column_name;
			}
		}
		$general_options = get_option( 'inbound_organizer_general');
		if ( $general_options ) {
			switch ( $general_options['sorting'] ) {
				case 2:
					$this->sorting = array( 'date' => 'asc' );
					break;
				default:
					$this->sorting = array( 'date' => 'desc' );
					break;
			}
		} else {
			$this->sorting = array( 'date' => 'desc' );
		}
		$this->posts_per_page = $general_options ? $general_options['page_size'] : 10;
		$this->color_classes = array(
									1 => 'inborg-gray',
									2 => 'inborg-red',
									3 => 'inborg-green',
									4 => 'inborg-blue',
									5 => 'inborg-white',
									6 => 'inborg-brown' );
	}

	/**
	 * Registers a custom post type and a custom post status for the plugin.
	 *
	 * This function defines a new post type for "Inbound Organizer Items" and 
	 * a custom post status called "Spam" with specific visibility rules.
	 */
	public function register_post_type() {
		register_post_type( $this->post_type, array(
			'labels' => array(
				'name' => __( 'Inbound Organizer Items', 'inbound-organizer' ),
				'singular_name' => __( 'Inbound Organizer Item', 'inbound-organizer' ),
			),
			'rewrite' => false,
			'query_var' => false,
		) );

		register_post_status( $this->spam_status, array(
			'label' => __( 'Spam', 'inbound-organizer' ),
			'public' => false,
			'exclude_from_search' => true,
			'show_in_admin_all_list' => false,
			'show_in_admin_status_list' => true,
		) );
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		 wp_enqueue_style( 'dashicons' );
		 wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/inbound-organizer-admin.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script('jquery-ui-core');
		wp_enqueue_script('jquery-ui-effects-core');
		wp_enqueue_script('jquery-ui-draggable');
		wp_enqueue_script('jquery-ui-droppable');
		wp_enqueue_script('jquery-ui-dialog');
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/inbound-organizer-admin.js', array( 'jquery' ), $this->version, false );
		wp_localize_script( $this->plugin_name, 'inbound_organizer_ajax_obj',
								array(
									'ajax_url' => admin_url( 'admin-ajax.php' ),
									'nonce'    => wp_create_nonce( 'organize messages' ),
								)
						  );
	}

	/**
	 * Adds a menu item and two subitems to the Dashboard menu.
	 *
	 * 1. The Inbound Organizer
	 * 2. The Settings for the Inbound Organizer
	 * @since    1.0.0
	 */
	public function add_menu () {
		add_menu_page( 
			'Inbound Organizer', 				//Title of the page as shown in the <title> tags
			'Inbound Organizer', 				//Name of the menu displayed on the Dashboard
			'manage_options', 					//Minimum capability required to view the menu
			'inbound_organizer_menu', 			//Slug name to refer to the menu
			array( $this, 'render_main_page'), 	//Function to be called to display page content.
			'dashicons-buddicons-pm',			//Icon URL or dashicons id for the icon to show
			99 									//Location in the menu where it should appear
		);
		add_submenu_page ( 'inbound_organizer_menu', 'Inbound Organizer', __('Organizer Settings', 'inbound-organizer' ), 'manage_options', 'inbound-organizer-settings-page', array( $this, 'render_settings_page'));
	}
	
	/**
	 * Renders one column with messages (cards) for the main page
	 *
	 * @since    1.0.0
	 * @param  	 integer   $status      	The tracking status for which the column must be rendered.
	 * @param    string    $status_name 	The version of this plugin.
	 * @param	 integer   $paged			The page number to be rendered. Default 1.
	 * @param	 boolean   $skip_container	Whether the container div must be rendered.
	 * @param	 boolean   $skip_header		Whether the header (h2) must be rendered.
	 * @param	 boolean   $check_load_more	Whether the load more button must be rendered if applicable.
	 * @return	 echo output in HTML 
	 */
	protected function render_main_page_column ( 
					$status, 
					$status_name, 
					$paged = 1, 
					$skip_container = false,
					$skip_heading = false,
					$check_load_more = true
		) {
		global $post;
		
		$query = new \WP_Query( 
			array( 
			'post_type' => $this->post_type,	
			'orderby' => $this->sorting,
			'posts_per_page' => $this->posts_per_page,
			'meta_key' => 'tracking_status',
			'meta_value' => $status,
			'meta_compare' => '=',
			'paged' => $paged
			) 
		);
		if (! $skip_container ) {
			$columns_options = get_option( 'inbound_organizer_columns' );
			$color_class = array_key_exists( 'column_color_'.$status, $columns_options ) 
							? $this->color_classes[ $columns_options['column_color_'.$status] ] 
							: $this->color_classes[1];
			?>
			<div class="inbound-organizer-column col-<?php echo count( $this->tracking_statuses ); ?>
						<?php echo esc_attr( $color_class ); ?>" 
						data-status="<?php echo esc_attr( $status ); ?>">
			<?php
		}
		if (! $skip_heading ) {
			?>
				<h2><?php echo esc_html( $status_name ).' ('.esc_html( $query->found_posts ).')'; ?></h2>
			<?php
		}
		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
						$card = $this->plugins->get_submission_data( $post->ID );
						//Skip card if submission data cannot be retrieved
						if ( ! $card ) continue;
						//If the color is set, use it. Otherwise white (color index 5)
						$color = get_post_meta( $post->ID, 'color', true );
						$color_class = ( empty ( $color ) ) 
										? $this->color_classes[5] 
										: $this->color_classes[$color];
		
						//Include partial with HTML
						require plugin_dir_path(  __FILE__  ) . 'partials/inbound-organizer-admin-column-view.php';
			}
			if ( $check_load_more ) {
				if ( $query->found_posts > ( $query->post_count + ( ( $paged -1 ) * $this->posts_per_page ) ) ) {
					$new_page = $paged + 1;
					echo '<button class="button-primary" data-status="'.esc_attr( $status ).'" data-page="'.esc_attr( $new_page ).'">'.esc_html( __('Load more', 'inbound-organizer') ).'</button>';
				}
			}
			$query->wp_reset_postdata();
		}
		if (! $skip_container ) {
			?>
			</div>
			<?php 
		}
		?>
		<div id="inbound-organizer-card-detail">
		</div>
		<div id="modal-background">
		</div>
	<?php
	}
	
	/**
	 * Renders the HTML lay-out for the Inbound Organizer and calls the function to render the columns
	 *
	 * @since    1.0.0
	 */
	public function render_main_page () {
		?>
		<div class="wrap">
			<h1 class="wp-heading-inline"><?php esc_html_e('Inbound Organizer', 'inbound-organizer'); ?></h1>
			<div class="inbound-organizer-container">
			<div id="inbound-organizer-message" class="notice notice-success is-dismissible">
			</div>
			<?php
				foreach ( $this->tracking_statuses as $code => $name) {
					$this->render_main_page_column( $code, $name );	
				} 
			?>
			</div>
		</div>
		<?php	
	}

	/**
	 * ========== AJAX FUNCTIONS ==========
	 **/

	/**
	 * Stores the new tracking status of a message.
	 * 
	 * When a messages is dragged & dropped from one column to another, the new status must be stored.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) for which the status must be updated.
	 * @param	string 	$status	 The new tracking status for the post (message).
	 * 
	 * Return via wp_send_json_success
	 * @return	array	JSON array containing a notification that the status has been updated.
	 */
	public function update_message_tracking_status () {
		check_ajax_referer( 'organize messages' );
		
		// Check if 'postData' exists and contains 'id' and 'status'
		if ( ! isset( $_POST['postData']['id'], $_POST['postData']['status'] ) ) {
			wp_send_json_error( array( 'message' =>  'Missing required parameters.' ) ); 		
		}

		$post_id = absint( $_POST['postData']['id'] );
		$status = strval( absint( $_POST['postData']['status'] ) );
		
		// Ensure $post_id is valid
		if ( ! get_post( $post_id ) ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) );
		}
		
		// Ensure status exists in tracking_statuses array
		if ( ! isset( $this->tracking_statuses[ $status ] ) ) {
			wp_send_json_error( array( 'message' => __( 'Invalid tracking status.', 'inbound-organizer' ) ) );
		}

		//Sanitizing and validating complete
		update_post_meta( $post_id, 'tracking_status', $status );

		// Return success response
		wp_send_json_success( array(
			'message' => esc_html( sprintf(
				/* translators: 1: Name field of the submission (as configured) 2: Target column name (= status) */ 
				__( 'Message from %1$s moved to %2$s.', 'inbound-organizer' ),
				$this->plugins->get_submission_data( $post_id )['name'],
				$this->tracking_statuses[ $status ] )
			)
		) );
	}

	/**
	 * Renders the HTML for the message popup.
	 * 
	 * When a messages is clicked, a popup is displayed with the details. 
	 * This function renders the HTML for that.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) for which the details must be rendered.
	 * 
	 * Return via wp_send_json_success
	 * @return	array	JSON array containing the rendered HTML.
	 */
	public function render_message_details () {
		check_ajax_referer( 'organize messages' );

		// Check if the required postData is set
		if ( ! isset( $_POST['postData']['id'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required post ID.' ) );
		}

		// Retrieve the post
		$post_id = absint( $_POST['postData']['id'] );
		$post = get_post( $post_id );
	
		if ( ! $post ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) );
		}

		$card = $this->plugins->get_submission_data( $post->ID );
		
		//Check if the display_config_form flag is set.
		$display_config_form = isset( $_POST['postData']['config'] ) ? absint ( $_POST['postData']['config'] )  : 0;	
	
		//Retrieve form field data if the user wants to change the layout or the field mapping is incomplete. 
		//This list is used to create dropdowns and store the values in the appropriate WP option field.  
		if ( ( $display_config_form == 1 ) || !$card['name'] || !$card['email'] || !$card['subject'] ) {
			$field_data = $this->plugins->get_field_data( $post->ID );
		}
		
		// Capture the HTML from the included partial
		ob_start();
		require plugin_dir_path( __FILE__ ) . 'partials/inbound-organizer-admin-card-view.php';
		$html_content = ob_get_clean(); // Get buffered content and clear the buffer
		
		//Send the response
		wp_send_json_success( array(
			'html'	=> $html_content
		) );
	}

	/**
	 * Marks a message (post) as Trash.
	 * 
	 * When a the user clicks on the corresponding icon,
	 * a messages must be marked as Trash so that it disappears from the column.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) to be marked as Trash.
	 * 
	 * Return via wp_send_json_success
	 * @return	array	JSON array containing a notification that the message has been sent to trash.
	 */
	public function trash_message() {
		check_ajax_referer( 'organize messages' );
		
		// Check if the required postData is set
		if ( ! isset( $_POST['postData']['id'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required post ID.' ) );
		}

		// Retrieve the post
		$post_id = absint( $_POST['postData']['id'] );
		$post = get_post( $post_id );
	
		if ( ! $post ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) ); 
		}

		wp_trash_post( $post_id );

		wp_send_json_success( array(
			'message' => esc_html( sprintf(
				/* translators: 1: Name field of the submission (as configured) */ 
				__( 'Message from %s sent to trash.', 'inbound-organizer' ), 
				$this->plugins->get_submission_data( $post_id )['name']
			) )
		) );
	}

	/**
	 * Saves notes for a message (post).
	 * 
	 * When the user adds notes to a message, these must be saved.
	 * These are stored in a seperate meta data field.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) to which the notes belong.
	 * 
	 * Return via wp_send_json 
	 * @return	array	JSON array with the post_id for which the notes were saved.
	 */
	public function save_message_notes() {
		check_ajax_referer( 'organize messages' );

		// Check if required post data exists
		if ( ! isset( $_POST['postData']['id'] ) || ! isset( $_POST['postData']['notes'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required data.' ) ); 
		}
	
		// Sanitize and validate post ID
		$post_id = absint( $_POST['postData']['id'] );
		if ( ! get_post( $post_id ) ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) );
		}

		$notes = sanitize_textarea_field( wp_unslash( $_POST['postData']['notes'] ) );
		update_post_meta( $post_id, 'notes', $notes);
		
		wp_send_json_success( array( 'result' => $post_id ) );
	}

	/**
	 * Saves layout for a specific form type based on changes in one message .
	 * 
	 * When a the user adds changes the layout of a message, these must be saved.
	 * The layout are stored in a plugin specific WP option.
	 * The layout is applied to all messages of this form type.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) in which the layout was configured.
	 * @param	string	$name	 The form field that must be displayed as name in messages of this form type.
	 * @param	string	$email	 The form field that must be displayed as email in messages of this form type.
	 * @param	string  $subject The form field that must be displayed as subject in messages of this form type.
	 * 
	 * Return via wp_send_json_success
	 * @return	array	JSON array containing a success message and the post_id.
	 */
	public function save_layout() {
		check_ajax_referer( 'organize messages' );
		
		// Ensure required field exist
		if ( ! isset( $_POST['postData']['id'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required data.' ) );
		}

		// Validate and sanitize inputs
		$args = array();
		$args['post_id'] = absint( $_POST['postData']['id'] );

		// Ensure post exists
		if ( ! get_post( $args['post_id'] ) ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) );
		}
		
		// Sanitize user input after removing added slashes
		$args['name'] = isset( $_POST['postData']['name'] ) ? sanitize_text_field( wp_unslash( $_POST['postData']['name'] ) ) : '';
		$args['email'] = isset( $_POST['postData']['email'] ) ? sanitize_text_field( wp_unslash( $_POST['postData']['email'] ) ) : '';
		$args['subject'] =  isset( $_POST['postData']['subject'] ) ? sanitize_text_field( wp_unslash( $_POST['postData']['subject'] ) ) : '';
		
		// Ensure at least one field is not empty
		if ( empty( $args['name'] ) && empty( $args['email'] ) && empty( $args['subject'] ) ) {
			wp_send_json_error( array( 'message' => 'At least one field (name, email, or subject) must be filled.' ) );
		}
	
		// Save layout data
		$result = $this->plugins->set_layout( $args );
		
		// Return response
		if ( $result ) {
			wp_send_json_success( array( 'message' => 'Layout saved successfully.', 'post_id' => $args['post_id'] ) );
		} else {
			wp_send_json_error( array( 'message' => 'Failed to save layout.' ) );
		}
	}

	/**
	 * Renders the HTML for additional messages in a column.
	 * 
	 * Messages are loaded per 'page' (subset). 
	 * This function loads the next page and renders the HTML for those messages.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$status The column for which the next page must be loaded.
	 * @param	inreger $page 	The page (number) that must be loaded
	 * 
	 * Return via wp_send_json_success 
	 * @return	array	JSON array containing the rendered HTML for the messages in the column.
	 */
	public function load_more_messages() {
		check_ajax_referer( 'organize messages' );

		// Ensure required fields exist
		if ( ! isset( $_POST['postData']['status'], $_POST['postData']['page'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required data.' ) );
		}
		$status = absint( $_POST['postData']['status'] );
		$page = absint( $_POST['postData']['page'] );

		ob_start();
		$this->render_main_page_column( $status, '', $page, true, true );
		$html_content = ob_get_clean(); // Get buffered content and clear the buffer
		
		//Send the response
		wp_send_json_success( array(
			'html'	=> $html_content
		) );
	}

	/**
	 * Re-renders the HTML for messages in a column.
	 * 
	 * When a message is moved, trashed or changed in color, the column with the message must
	 * be refreshed to reflect the change.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$status The column for which the messages must be refreshed.
	 * @param	integer $count 	The number of messages in the column to be refreshed.
	 * 
	 * Return via wp_send_json_success
	 * @return	array	JSON array containing the rendered HTML for the messages in the column.
	 */
	public function refresh_messages() {
		check_ajax_referer( 'organize messages' );
		
		// Ensure required fields exist
		if ( ! isset( $_POST['postData']['status'], $_POST['postData']['count'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required data.' ) ); 
		}

		$status = absint( $_POST['postData']['status'] );
		$count = absint( $_POST['postData']['count'] );
		$number_of_pages = (int) ( ( $count - 1) / $this->posts_per_page ) + 1;

		ob_start();
		for ( $page = 1; $page <= $number_of_pages; $page++ ) {
			$this->render_main_page_column( 
				$status, 
				$this->tracking_statuses[ $status ], 
				$page, 
				true, 									//Skip container rendering (div)
				( $page > 1 ), 							//Skip heading (h2)
				( $page == $number_of_pages ) 			//Display load more button if applicable
			);			
		}
		$html_content = ob_get_clean(); // Get buffered content and clear the buffer
		
		//Send the response
		wp_send_json_success( array(
			'html'	=> $html_content
		) );
	}
	
	/**
	 * Updates the display color for a message.
	 * 
	 * When a the user changes the display color, this must be saved.
	 * It is stored in a seperate meta data field.
	 *
	 * @since    1.0.0
	 * 
	 * Input via $_POST
	 * @param 	integer	$post_id The post (message) for which the color was changed.
	 * @param	integer $color	 The color index that was selected.	
	 * 
	 * Return via wp_send_json_success 
	 * @return	array	JSON array containing the post id and the class name of the color that was saved.
	 */
	public function update_message_color() {
		check_ajax_referer( 'organize messages' );
		// Ensure required fields exist
		if ( ! isset( $_POST['postData']['id'], $_POST['postData']['color'] ) ) {
			wp_send_json_error( array( 'message' => 'Missing required data.' ) );
		}

		$post_id = absint ( $_POST['postData']['id'] );
		// Ensure $post_id is valid
		if ( ! get_post( $post_id ) ) {
			wp_send_json_error( array( 'message' => 'Invalid post ID.' ) );
		}

		$color = absint ( $_POST['postData']['color'] );

		// Ensure $color is valid
		if ( ! array_key_exists( $color, $this->color_classes ) )  {
			wp_send_json_error( array( 'message' => 'Invalid color.' ) );
		}
		
		update_post_meta( $post_id, 'color', $color );
		
		wp_send_json_success( array( 
						'color_class' => $this->color_classes[$color],
						'post_id' => $post_id
						   ) 
					);
	}
	/**
	 * ============= END AJAX =============
	 */

	/**
	 * ============= SETTINGS =============
	 */
	
	/**
	 * Register settings, settings sections and fields.
	 * 
	 * There are 2 setting sections: 1 for column settings and 1 for general settings.
	 * 
	 * @since    1.0.0
	 */
	public function init_settings() {
		
		//Register column settings
        register_setting( 
			'inbound_organizer_columns' , 
			'inbound_organizer_columns' , 
			array (
				'type' => 'string',
				'sanitize_callback' => array( $this, 'sanitize_columns_settings' ),
				'default' => NULL
			)
		);
        add_settings_section ( 
			'inbound_organizer_columns',       				//Section name
            __( 'Columns', 'inbound-organizer' ),      		//Title displayed
            array( $this, 'render_explanation_columns' ),   //Callback to echo additional explanation
            'inbound-organizer-columns-settings' 			//Page name
		);         
        
		//Create a field for the number of columns
        add_settings_field ( 
			'inbound_organizer_number_of_columns',          //ID
            __('Number of columns', 'inbound-organizer' ),  //Field label
            array( $this, 'render_number_of_columns'),  	//Callback to render field
            'inbound-organizer-columns-settings',           //Page name
            'inbound_organizer_columns');                  	//Section name

		//Create fields for the column names and colors
		$columns = get_option( 'inbound_organizer_columns' )['number_of_columns'];
		for ( $column = 1; $column <= $columns; $column++ ) {
			add_settings_field ( 
				'inbound_organizer_column_name_'.$column,							//ID
				/* translators: 1: column number */	          				  		
                sprintf( __( 'Column name %d', 'inbound-organizer' ), $column ), 	//Field label
                array( $this, 'render_column_name'),  								//Callback to render field
                'inbound-organizer-columns-settings',       					  	//Page name
                'inbound_organizer_columns',									  	//Section name
				array( 'column' => $column ) 										//Pass column number to callback
			);    					
			add_settings_field ( 
				'inbound_organizer_column_color_'.$column,          				//ID
				/* translators: 1: column number */
				sprintf( __( 'Column color %d', 'inbound-organizer' ), $column ),	//Field label
				array( $this, 'render_column_color'),  								//Callback to render field
				'inbound-organizer-columns-settings',              					//Page name
				'inbound_organizer_columns',										//Section name
				array( 'column' => $column ) 										//Pass column number to callback	
			);
		}              	
		
		//Register general settings
        register_setting( 
			'inbound_organizer_general' , 
			'inbound_organizer_general' , 
			array (
				'type' => 'string',
				'sanitize_callback' => array( $this, 'sanitize_general_settings' ),
				'default' => NULL
			)
		);
		add_settings_section ( 
			'inbound_organizer_general',       				//Section name
            __( 'General', 'inbound-organizer' ),      		//Title displayed
            array( $this, 'render_explanation_general' ),   //Callback to echo additional explanation
            'inbound-organizer-general-settings' 			//Page name
		);

		//Create a field for the number of columns
        add_settings_field ( 
			'inbound_organizer_sorting',             		//ID
            __('Sorting', 'inbound-organizer' ),		    //Field label
            array( $this, 'render_sorting'),  				//Callback to render field
            'inbound-organizer-general-settings',           //Page name
            'inbound_organizer_general'						//Section name
		
		);                  	

		//Create a field for the number of messsages per page
        add_settings_field ( 
			'inbound_organizer_page_size',             			//ID
            __('Page size (# messages)', 'inbound-organizer' ),	//Field label
            array( $this, 'render_page_size'),  				//Callback to render field
            'inbound-organizer-general-settings',              	//Page name
            'inbound_organizer_general'							//Section name
		);
	}

	/**
	 * Callback function for settings menu. 
	 * 
	 * It will echo the HTML for the settings page consisting of 3 tabs:
	 * 1. General
	 * 2. Column
	 * 
	 * @since    1.0.0
	 */
	public function render_settings_page () {	
		if ( current_user_can( 'manage_options' ) ) {
			?>
			<div class="wrap">
				<h1 class="wp-heading-inline"><?php esc_html_e( 'Inbound Organizer Settings', 'inbound-organizer' ); ?></h1>
				<?php

				//Get the active tab from the $_GET param
				//This $_GET parameter only controls display logic and therefore a nonce is not required
  				$tab = isset( $_GET['tab'] ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : null;
				?>  
				<nav class="nav-tab-wrapper">
				<a href="?page=inbound-organizer-settings-page" class="nav-tab 
				  	<?php if ( $tab === null ) { ?>nav-tab-active<?php } ?>">
					<?php echo esc_html( __( 'General', 'inbound-organizer' ) ); ?></a>
				<a href="?page=inbound-organizer-settings-page&tab=columns" class="nav-tab 
				  	<?php if ( $tab === 'columns' ) { ?>nav-tab-active<?php } ?>">
					<?php echo esc_html( __( 'Columns', 'inbound-organizer' ) ); ?></a>
				</nav>
				<div class="tab-content">
    			<?php switch ( $tab ) {
					case 'columns':
						settings_errors( 'inbound_organizer_columns' );
						?>
						<form action="options.php" method="post">
							<?php
							settings_fields( 'inbound_organizer_columns' );
							do_settings_sections( 'inbound-organizer-columns-settings' );
							submit_button( 
								__( 'Save changes', 'inbound-organizer' ), 
								'primary' 
							);
							?>
						</form>
						<?php
						break;
					default:
						settings_errors( 'inbound_organizer_general' );
						?>
						<form action="options.php" method="post">
							<?php
							settings_fields( 'inbound_organizer_general' );
							do_settings_sections( 'inbound-organizer-general-settings' );
							submit_button( 
								__( 'Save changes', 'inbound-organizer' ), 
								'primary' 
							);
							?>
						</form>
						<?php
						break;
					};
				?>
				</div>
			</div>
			<?php	
		}
	}

	/**
     * Callback function to display introductory text for columns settings page.
     *
	 * @since    1.0.0
     */
    public function render_explanation_columns() {
		echo '<p>'.esc_html( __( 'Change the column settings here.', 'inbound-organizer' ) ).'</p>';
	 }

	 /**
     * Callback function to display introductory text for general settings page.
     *
	 * @since    1.0.0 
     */
    public function render_explanation_general() {
		echo '<p>'.esc_html( __( 'Change the general settings here.', 'inbound-organizer' ) ).'</p>';
	 }

	/**
     * Callback function to sanitize columns settings after these have been submitted.
     *
	 * @since    1.0.0
	 * 
     * @param array $input Submitted setting values as key => value
     * 
     * @return array $valid Sanitized setting values as key => value
     */
    public function sanitize_columns_settings ( $input ) {    
		$valid = array();
		$options = get_option( 'inbound_organizer_columns' );

        //Number_of_columns between 2 and 5 (automatically corrected)
		$valid['number_of_columns'] = absint( $input['number_of_columns'] );
		$valid['number_of_columns'] = ( $valid['number_of_columns'] < 2 ) 
										? 2 : $valid['number_of_columns'];
		$valid['number_of_columns'] = ( $valid['number_of_columns'] > 5 ) 
										? 5 : $valid['number_of_columns'];

		//The column_name must be max 15 characters (with a default for empty strings), column_color between 1 and 6 (automatically corrected)
		for ( $column = 1; $column <= $valid['number_of_columns']; $column++ ) {
			//Trim name to 15 characters and sanitize
			$valid['column_name_'.$column] = array_key_exists( 'column_name_'.$column, $input )
					? sanitize_text_field( substr( $input['column_name_'.$column], 0, 15 ) ) : '';
			//Assign default if empty 
			if ( empty( $valid['column_name_'.$column] ) ) {
				$valid['column_name_'.$column] = __( 'Column', 'inbound-organizer' ).' '.$column;
			}
			//Convert to positive integer and default to value 1
			$valid['column_color_'.$column] = array_key_exists( 'column_color_'.$column, $input )
				? absint( $input['column_color_'.$column] ) : 1;
			//The value must be at least 1
			$valid['column_color_'.$column] = ( $valid['column_color_'.$column] < 1 ) 
												? 1 : $valid['column_color_'.$column];
			//The value must be at most 6
			$valid['column_color'] = ( $valid['column_color_'.$column] > 6 ) 
										? 6 : $valid['column_color_'.$column];        
		}
        return $valid;
    }

	/**
	 * Renders the settings field for the number of columns.
	 * 
	 * If no setting is stored, it defaults to 3.
	 * 
	 * @since    1.0.0
	 */
	public function render_number_of_columns() {
		$options = get_option( 'inbound_organizer_columns' );
        $number_of_columns = array_key_exists ( 'number_of_columns', $options ) 
								? absint( $options['number_of_columns'] ) : 3 ;
        $columns = array (  
			2 => __( '2 columns', 'inbound-organizer' ), 
            3 => __( '3 columns', 'inbound-organizer'),
			4 => __( '4 columns', 'inbound-organizer'),
			5 => __( '5 columns', 'inbound-organizer')
        );

        //Initiate the dropdown field and mark the current value as selected
        echo '<select id="number_of_columns" name="inbound_organizer_columns[number_of_columns]">';
        foreach ( $columns as $column => $description ) {
            echo '<option value="'.esc_attr( $column ).'" '
                 .selected( $number_of_columns, $column, false ).'>'
                 .esc_html( $description ).'</option>';
        }
        echo "</select>";
	}

	/**
	 * Renders the settings field for the column name.
	 * Defaults to 'Column [column number]'.
	 * 
	 * @param array $args	Contains additional arguments. Currently filled with one value: column, which is the column number.
	 * 
	 * @since    1.0.0
	 */
	public function render_column_name( array $args ) {
		$options = get_option( 'inbound_organizer_columns' );
        $name = array_key_exists( 'column_name_'.$args['column'], $options ) 
					? $options['column_name_'.$args['column']] 
					: '';
		$name = empty( $name )
				/* translators: 1: column number */ 
				? sprintf( __( 'Column %s', 'inbound-organizer' ), $args['column'] )
				: $name;
		echo '<input type="text" size="15" maxlength="15" id="column_name_'.
			  esc_attr( $args['column'] ).'" 
			  name="inbound_organizer_columns[column_name_'.esc_attr( $args['column'] ).']" 
			  value="'.esc_attr( $name ).'" />';
	}

	/**
	 * Renders the settings field for the column color
	 * Defaults to color index 1, Gray
	 * 
	 * @param array $args	Contains additional arguments. Currently filled with one value: column, which is the column number.
	 * 
	 * @since	1.0.0
	 */
	public function render_column_color( array $args ) {
		$options = get_option( 'inbound_organizer_columns' );
        $color = array_key_exists( 'column_color_'.$args['column'], $options ) ? 
				$options['column_color_'.$args['column']] : 1;
		$modes = array ( 
				1 => __( 'Gray', 'inbound-organizer' ), 
				2 => __( 'Red', 'inbound-organizer'),
				3 => __( 'Green', 'inbound-organizer'),
				4 => __( 'Blue', 'inbound-organizer'),
				6 => __( 'Brown', 'inbound-organizer')
		);

		//Initiate the dropdown field and mark the current value as selected
		echo '<select id="column_color_'.esc_attr( $args['column'] ).
				'" name="inbound_organizer_columns[column_color_'.
				esc_attr( $args['column'] ).']">';
		foreach ( $modes as $mode => $description ) {
		echo '<option value="'.esc_attr( $mode ).'" '
				.selected( $color, $mode, false ).'
				class="'.esc_attr( $this->color_classes[$mode] ).'">'
				.esc_html( $description ).'</option>';
		}
		echo "</select>";
	}

	/**
     * Callback function to sanitize general settings after these have been submitted.
     * 
     * @param array $input Submitted setting values as key => value
     * 
     * @return array $valid Sanitized setting values as key => value
	 * 
	 * @since	1.0.0
     */
	public function sanitize_general_settings ( $input ) {

		//The sorting between 1 and 3 (automatically corrected)
		$valid['sorting'] = absint( $input['sorting'] );
		$valid['sorting'] = ( $valid['sorting'] < 1 ) ? 1 : $valid['sorting'];
		$valid['sorting'] = ( $valid['sorting'] > 2 ) ? 2 : $valid['sorting'];
        if ( $valid['sorting'] != $input['sorting'] ) {
            add_settings_error(
                'inbound_organizer_general',
                'inbound_organizer_invalid_sorting', 
                'The sorting setting has been adjusted to fit in range.',
                'update'
            );
        }

		//The page_size one of the allowed values 5, 10, 15 or 20, corrected to 10 if needed
		$valid['page_size'] = in_array( absint( $input['page_size'] ), array( 5, 10, 15, 20) ) 
							? absint( $input['page_size'] ): 10;
		if ( $valid['page_size'] != $input['page_size'] ) {
			add_settings_error(
				'inbound_organizer_general',
				'inbound_organizer_invalid_page_size', 
				'The page size setting has been adjusted to fit in range.',
				'update'
			);
		}
		return $valid;

	}
	/**
	 * Renders the settings field for sorting.
	 * Defaults to 1, Date / time descending.
	 * 
	 * @since	1.0.0
	 */
	public function render_sorting() {
		$options = get_option( 'inbound_organizer_general' );
        $sorting = array_key_exists( 'sorting', $options) ? 
					$options['sorting'] : '';
		$sorting = empty ( $sorting ) ? 1 : $sorting;
        $modes = array ( 
			1 => __( 'Date / time descending', 'inbound-organizer' ), 
            2 => __( 'Date / time ascending', 'inbound-organizer')
        );

        //Initiate the dropdown field and mark the current value as selected
        echo '<select id="sorting" name="inbound_organizer_general[sorting]">';
        foreach ( $modes as $mode => $description ) {
            echo '<option value="'.esc_attr( $mode ).'" '
                 .selected( $sorting, $mode, false ).'>'
                 .esc_html( $description ).'</option>';
        }
        echo "</select>";
	}

	/**
	 * Renders the settings field for the page size.
	 * Defaults to 10.
	 * 
	 * @since	1.0.0
	 */
	public function render_page_size() {
		$options = get_option( 'inbound_organizer_general' );
        $page_size = array_key_exists( 'page_size', $options) ? 
					$options['page_size'] : '';
		$page_size = empty ( $page_size ) ? 10 : $page_size;
        $modes = array ( 
			5 => '5', 
            10 => '10',
			15 => '15',
			20 => '20'
        );

        //Initiate the dropdown field and mark the current value as selected
        echo '<select id="page_size" name="inbound_organizer_general[page_size]">';
        foreach ( $modes as $mode => $description ) {
            echo '<option value="'.esc_attr( $mode ).'" '
                 .selected( $page_size, $mode, false ).'>'
                 .esc_html( $description ).'</option>';
        }
        echo "</select>";
	}
}
