<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Define the interface for the plugin specific methods
 */
interface Form_Plugin {
	public function add_message( $args );
	public function get_submission_data ( $id );
	public function get_fields_inbound( $form );
	public function get_forms();
	public function set_layout( $form, $args );
	public function get_layout( $form );
}


/**
 * The functionality to abstract the different form plugin data structures.
 * It creates an open architecture that can accommodate many morge form plugins.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 *
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */

 class Inbound_Organizer_Plugins {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The post type to store data about form submissions.
	 *
	 * @since    1.0.0
	 * @var      string    $post_type    The post type that will be queried and displayed.
	 */
	public $post_type;

	/**
	 * The post status to indicate a spam message.
	 *
	 * @since    1.0.0
	 * @var      string    $spam_status   The post status used to indicate spam.
	 */
	public $spam_status;
	
	/**
	 * Instance of CFDB7 integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_CFDB7 $ninjaforms Object instance.   
	 */
	private $cfdb7;

	/**
	 * Instance of Flamingo integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Flamingo $flamingo Object instance.   
	 */
	private $flamingo;

	/**
	 * Instance of Fluent Forms integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_FluentForms $fluentforms Object instance.   
	 */
	private $fluentforms;

	/**
	 * Instance of Formidable integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Formidable $formidable Object instance.   
	 */
	private $formidable;
	
	/**
	 * Instance of Forminator integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Forminator $forminator Object instance.   
	 */
	private $forminator;

	/**
	 * Instance of Gravity Forms integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_GravityForms $gravityforms Object instance.   
	 */
	private $gravityforms;

	/**
	 * Instance of Ninja Forms integration class
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_NinjaForms $ninjaforms Object instance.   
	 */
	private $ninjaforms;

	/**
	 * Which format to use for dates displayed
	 *
	 * @since    1.0.0
	 * @var      string    $date_format   A string in the common notation for date formatting.
	 */
	public $date_format;

	/**
	 * Which format to use for times displayed
	 *
	 * @since    1.0.0
	 * @var      string    $time_format   A string in the common notation for time formatting.
	 */
	public $time_format;
	
	/**
	 * Class constructor for initializing the plugin object.
	 *
	 * This constructor sets up the plugin's core properties, including its name,
	 * version, custom post type, spam status meta key, and date/time formats.
	 *
	 * @since 1.0.0
	 *
	 * @param string $plugin_name The unique name of the plugin.
	 * @param string $version     The current version of the plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		// Store the plugin's unique name for later reference.
		$this->plugin_name = $plugin_name;

		// Store the plugin's current version for version-specific functionality.
		$this->version = $version;

		// Define a custom post type identifier for items managed by the plugin.
		$this->post_type = 'inborg_item';

		// Define a post status to indicate a spam message.
		$this->spam_status = 'inborg_spam_status';

		// Retrieve and store the date format option from WordPress settings.
		$this->date_format = get_option( 'date_format' );

		// Retrieve and store the time format option from WordPress settings.
		$this->time_format = get_option( 'time_format' );
	}

	/**
	 * Converts a date to a more user-friendly string ('Today' or 'Yesterday') when applicable.
	 *
	 * This helper function checks if the given date corresponds to today or yesterday.
	 * If so, it replaces the date string with 'Today' or 'Yesterday'. Otherwise, the
	 * original date string is returned unchanged.
	 *
	 * @since 1.0.0
	 *
	 * @param WP_Post $post The post object containing the date to compare.
	 * @param string  $date The original date string to be potentially modified.
	 *
	 * @return string The modified or original date string.
	 */
	public function friendly_date( $post, $date ) {
		// Create a DateTime object for the post's date, set to midnight for accurate comparison.
		$compare_date = new \DateTime( $post->post_date );
		$compare_date->setTime( 0, 0, 0 );

		// Create a DateTime object for the current date, also set to midnight.
		$now = new \DateTime();
		$now->setTime( 0, 0, 0 );

		// Calculate the difference in days between the current date and the post date.
		switch ( $now->diff( $compare_date )->days ) {
			case 0:
				// If the difference is 0 days, the date refers to today.
				$date = __( 'Today', 'inbound-organizer' );
				break;

			case 1:
				// If the difference is 1 day, the date refers to yesterday.
				$date = __( 'Yesterday', 'inbound-organizer' );
				break;

			default:
				// For all other cases, retain the original date string.
				break;
		}

		// Return the modified or original date string.
		return $date;
	}


	/**
	 * Searches for a substring within an array of strings and returns the first matching string.
	 *
	 * This helper function iterates through the provided array and checks if the given substring is present
	 * in any of the strings. If found, it returns the first matching string. If no match is found, it returns false.
	 *
	 * @since 1.0.0
	 *
	 * @param string $substring The substring to search for.
	 * @param array  $array     The array of strings to search within.
	 *
	 * @return string|false The first string containing the substring, or false if no match is found.
	 */
	public function search_substring_in_array( $substring, $array ) {
		// Iterate through each string in the array.
		foreach ( $array as $string ) {
			// Check if the substring is found in the current string.
			if ( strpos( $string, $substring ) !== false ) {
				// Return the string in which the substring was found.
				return $string;
			}
		}

		// Return false if the substring is not found in any string.
		return false;
	}


	/**
	 * Sets up the Flamingo plugin integration.
	 *
	 * This function initializes the integration with the Flamingo plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Flamingo integration is successfully set up, false otherwise.
	 */
	protected function set_flamingo() { 
		// Check if the Flamingo integration has already been set up.
		if (! isset( $this->flamingo ) ) { 
			//Check if the Flamingo plugin is active.
			if ( is_plugin_active( 'flamingo/flamingo.php' ) ) {
				// Include the file that defines the Flamingo integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/flamingo.php';
				
				// Instantiate the Flamingo integration class and store it in the object property.
				$this->flamingo = new Inbound_Organizer_Flamingo( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		} 

		// Return true if the Flamingo integration is set up successfully.
		return true;
	}

	/**
	 * Sets up the CFDB7 plugin integration.
	 *
	 * This function initializes the integration with the CFDB7 plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the CFDB7 integration is successfully set up, false otherwise.
	 */
	protected function set_cfdb7() {
		// Check if the CFDB7 integration has already been set up.
		if (! isset( $this->cfdb7 ) ) {
			//Check if the CFDB7 plugin is active.
			if ( is_plugin_active( 'contact-form-cfdb7/contact-form-cfdb-7.php') ) { 
				// Include the file that defines the CFDB7 integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/cfdb7.php';
				
				// Instantiate the CFDB7 integration class and store it in the object property.
				$this->cfdb7 = new Inbound_Organizer_CFDB7( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}

		// Return true if the CFDB7 integration is set up successfully.
		return true;				
	}

	/**
	 * Sets up the Fluent Forms plugin integration.
	 *
	 * This function initializes the integration with the Fluent Forms plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Fluent Forms integration is successfully set up, false otherwise.
	 */
	protected function set_fluentforms() {
		// Check if the Fluent Forms integration has already been set up.
		if (! isset( $this->fluentforms ) ) {
			//Check if the Fluent Forms plugin is active.
			if ( is_plugin_active( 'fluentform/fluentform.php' ) ) {
				// Include the file that defines the Fluent Forms integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/fluentforms.php';
				
				// Instantiate the Fluent Forms integration class and store it in the object property.
				$this->fluentforms = new Inbound_Organizer_FluentForms( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}

		// Return true if the Fluent Forms integration is set up successfully.
		return true;
	}

	/**
	 * Sets up the Gravity Forms plugin integration.
	 *
	 * This function initializes the integration with the Gravity Forms plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Gravity Forms integration is successfully set up, false otherwise.
	 */
	protected function set_gravityforms() {
		// Check if the Gravity Forms integration has already been set up.
		if (! isset( $this->gravityforms ) ) {
			//Check if the Gravity Forms plugin is active.
			if ( is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
				// Include the file that defines the Gravity Forms integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/gravityforms.php';
				
				// Instantiate the Gravity Forms integration class and store it in the object property.
				$this->gravityforms = new Inbound_Organizer_GravityForms( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}

		// Return true if the Gravity Forms integration is set up successfully.
		return true;
	}

	/**
	 * Sets up the Formidable plugin integration.
	 *
	 * This function initializes the integration with the Formidable plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Formidable integration is successfully set up, false otherwise.
	 */
	protected function set_formidable() {
		// Check if the Formidable integration has already been set up.
		if (! isset( $this->formidable ) ) {
			//Check if the Formidable plugin is active.
			if ( is_plugin_active( 'formidable/formidable.php' ) ) {
				// Include the file that defines the Formidable integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/formidable.php';

				// Instantiate the Formidable integration class and store it in the object property.
				$this->formidable = new Inbound_Organizer_Formidable( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}

		// Return true if the Formidable integration is set up successfully.
		return true;
	}

	/**
	 * Sets up the Forminator plugin integration.
	 *
	 * This function initializes the integration with the Forminator plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Forminator integration is successfully set up, false otherwise.
	 */
	protected function set_forminator() {
		// Check if the Forminator integration has already been set up.
		if (! isset( $this->forminator ) ) {
			//Check if the Forminator plugin is active.
			if ( is_plugin_active( 'forminator/forminator.php' ) ) {
				// Include the file that defines the Forminator integration class.
				require_once plugin_dir_path(  __FILE__  ) . 'supported-plugins/forminator.php';
				
				// Instantiate the Forminator integration class and store it in the object property.
				$this->forminator = new Inbound_Organizer_Forminator( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}

		// Return true if the Forminator integration is set up successfully.
		return true;
	}

	/**
	 * Sets up the Ninja Forms plugin integration.
	 *
	 * This function initializes the integration with the Ninja Forms plugin by checking if the plugin is active,
	 * including the required integration file, and instantiating the corresponding class.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the Ninja Forms integration is successfully set up, false otherwise.
	 */
	protected function set_ninjaforms() {
		// Check if the Ninja Forms integration has already been set up.
		if ( ! isset( $this->ninjaforms ) ) {
			// Check if the Ninja Forms plugin is active.
			if ( is_plugin_active( 'ninja-forms/ninja-forms.php' ) ) {
				// Include the file that defines the Ninja Forms integration class.
				require_once plugin_dir_path( __FILE__ ) . 'supported-plugins/ninjaforms.php';
	
				// Instantiate the Ninja Forms integration class and store it in the object property.
				$this->ninjaforms = new Inbound_Organizer_NinjaForms( $this );
			} else {
				// Return false if the plugin is not active.
				return false;
			}
		}
	
		// Return true if the Ninja Forms integration is set up successfully.
		return true;
	}


	/**
	 * Checks which supported plugins are active and registers them for integration.
	 *
	 * This function scans a predefined list of supported plugins to determine which are active. 
	 * For each active plugin, it initializes the required setup and registers hooks to capture 
	 * data entities (e.g., forms and submissions) and their associated fields. 
	 *
	 * @since 1.0.0
	 *
	 * @return void This function does not return a value but performs plugin setup and integration.
	 */
	public function check_plugins () {
		$plugin_list = [
			'flamingo/flamingo.php' => 'Flamingo',
			'contact-form-cfdb7/contact-form-cfdb-7.php' => 'Contact Form CFDB7',
			'fluentform/fluentform.php' => 'Fluent Forms',
			'formidable/formidable.php' => 'Formidable Forms',
			'forminator/forminator.php' => 'Forminator',
			'ninja-forms/ninja-forms.php' => 'Ninja Forms',
			'gravityforms/gravityforms.php' => 'Gravity Forms'
		];
		$plugin_base_path = trailingslashit( WP_PLUGIN_DIR );
		$active_plugins = get_option('active_plugins');
		foreach ( $plugin_list as $plugin_key => $plugin_name ) {
			 // Check if the plugin is active.
			if ( in_array( $plugin_key, $active_plugins ) ) {
				$details = array();
				switch ( $plugin_key ) {
					case 'flamingo/flamingo.php':
						$this->set_flamingo();
						add_filter( 'flamingo_add_inbound', array( $this->flamingo, 'add_message' ) );
						break;
					case 'contact-form-cfdb7/contact-form-cfdb-7.php':
						$this->set_cfdb7();
						add_action( 'cfdb7_after_save_data', array( $this->cfdb7, 'add_message') );
						break;
					case 'fluentform/fluentform.php':
						$this->set_fluentforms();
						add_action( 'fluentform/submission_inserted', function( $submissionId, $formData, $form ) {
								$this->fluentforms->add_message( [ 'submissionId' => $submissionId, 'formData' => $formData, 'form' => $form ] );
						}, 10, 3);
						break;
					case 'gravityforms/gravityforms.php':
						$this->set_gravityforms();
						add_action( 'gform_after_submission', function ( $entry, $form ) {
							$this->gravityforms->add_message( [ 'id' => $entry['id'], 'form_id' => $entry['form_id'] ] );
						}, 10, 2 );
						break;
					case 'formidable/formidable.php':
							$this->set_formidable();
							add_action( 'frm_after_create_entry', function ( $entry, $form ) {
								$this->formidable->add_message( [ 'id' => $entry, 'form_id' => $form ] );
							}, 10, 2 );
							break;
					case 'forminator/forminator.php':
							$this->set_forminator();
							add_action( 'forminator_form_after_save_entry', function ( $form_id, $response ) {
								if ( class_exists('Forminator_Form_Entry_Model') ) {
									$entry = \Forminator_Form_Entry_Model::get_latest_entry_by_form_id( $form_id );
									$this->forminator->add_message( [ 'id' => $entry->entry_id, 'form_id' => $form_id ] );
								}
							}, 10, 2 );
							add_action( 'forminator_form_after_handle_submit', function ( $form_id, $response ) {
								if ( class_exists('Forminator_Form_Entry_Model') ) {
									$entry = \Forminator_Form_Entry_Model::get_latest_entry_by_form_id( $form_id );
									$this->forminator->add_message( [ 'id' => $entry->entry_id, 'form_id' => $form_id ] );
								}
							}, 10, 2 );
							break;
					case 'ninja-forms/ninja-forms.php':
						$this->set_ninjaforms();
						add_action( 'ninja_forms_after_submission', function ( $form_data ) {
							$form_id = $form_data['form_id'];
							if ( class_exists( 'Ninja_Forms' )  ) {
								$args = array(
									'posts_per_page' => 1,
									'order' => 'DESC',
									'orderby' => 'date'
								);
								$submissions = Ninja_Forms()->form( $form_id )->get_subs();
								$submission_id = array_key_first( $submissions );
								$this->ninjaforms->add_message( [ 'id' => $submission_id, 'form_id' => $form_id ] );
							}
						}, 10, 1 );
						break;
				}
			}
		}
		return;
	}

	/**
	 * Retrieves submission data for a form based on its originating plugin (source) and the corresponding submission ID (source_id).
	 *
	 * This function determines the submission source plugin (e.g., Flamingo, FluentForms, GravityForms, etc.)
	 * and retrieves the corresponding submission data. If applicable, the function calculates
	 * the submission time and date based on the WordPress timezone settings.
	 *
	 * @param int $post_id The ID of a post of the type inborg_item that represents a form submission in Inbound Organizer.
	 *
	 * @return array {
	 *     An associative array containing submission data and metadata.
	 *
	 *     @type string $name    The value of the field that is mapped to the card item 'Name'.
	 *     @type string $email   The value of the field that is mapped to the card item 'Email'.
	 *     @type string $subject The value of the field that is mapped to the card item 'Subject'.
	 *     @type string $time    The time of the submission formatted according to the
	 *                           plugin's configured time format.
	 *     @type string $date    The date of the submission formatted according to the
	 *                           plugin's configured date format.
	 *     @type array  $fields  Field values of the submission for non-empty fields only.
	 * }
	 */
	public function get_submission_data( $post_id ) {
		$source = get_post_meta( $post_id, 'source' )[0];
		$source_id = get_post_meta ( $post_id, 'source_id' )[0];
		$wp_timezone = wp_timezone();
		switch ( $source ) {
			case 'flamingo':
				if ( $this->set_flamingo() ) {
					$result = $this->flamingo->get_submission_data( $source_id );
					$post = get_post( $post_id );
					$post_timestamp = get_post_timestamp( $post );
					$date = new \DateTime( "@$post_timestamp" );
					$date->setTimezone( $wp_timezone );
					$result['time'] = $date->format( $this->time_format );
					$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
				} else $result = false;
				break;
			case 'cfdb7':
				if ( $this->set_cfdb7() ) {
					$result = $this->cfdb7->get_submission_data( $source_id );
					$post = get_post( $post_id );
					$post_timestamp = get_post_timestamp( $post );
					$date = new \DateTime( "@$post_timestamp" );
					$date->setTimezone( $wp_timezone );
					$result['time'] = $date->format( $this->time_format );
					$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
				} else $result = false;
				break;
			case 'fluentform':
				if ( $this->set_fluentforms() ) {
					$result = $this->fluentforms->get_submission_data( $source_id );
					$post = get_post( $post_id );
					$post_timestamp = get_post_timestamp( $post );
					$date = new \DateTime( "@$post_timestamp" );
					$date->setTimezone( $wp_timezone );
					$result['time'] = $date->format( $this->time_format );
					$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
				} else $result = false;
				break;
			case 'gravityforms':
				if ( $this->set_gravityforms() ) {
					$result = $this->gravityforms->get_submission_data( $source_id );
					$post = get_post( $post_id );
					$post_timestamp = get_post_timestamp( $post );
					$date = new \DateTime( "@$post_timestamp" );
					$date->setTimezone( $wp_timezone );
					$result['time'] = $date->format( $this->time_format );
					$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
				} else $result = false;
				break;
			case 'formidable':
					if ( $this->set_formidable() ) {
						$result = $this->formidable->get_submission_data( $source_id );
						$post = get_post( $post_id );
						$post_timestamp = get_post_timestamp( $post );
						$date = new \DateTime( "@$post_timestamp" );
						$date->setTimezone( $wp_timezone );
						$result['time'] = $date->format( $this->time_format );
						$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
					} else $result = false;
					break;
			case 'forminator':
					if ( $this->set_forminator() ) {
						//Forminator needs the form_id and the source_id to retrieve an entry
						//Therefore we also retrieve the form_type form the post
						$form_id = get_post_meta ( $post_id, 'form_type' )[0];
						$result = $this->forminator->get_submission_data( [ 'entry_id' => $source_id, 'form_id' => $form_id ] );
						$post = get_post( $post_id );
						$post_timestamp = get_post_timestamp( $post );
						$date = new \DateTime( "@$post_timestamp" );
						$date->setTimezone( $wp_timezone );
						$result['time'] = $date->format( $this->time_format );
						$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
					} else $result = false;
					break;
			case 'ninjaforms':
					if ( $this->set_ninjaforms() ) {
						//Ninjaforms needs the form_id to retrieve the field labels of a submission
						//Therefore we also retrieve the form_type form the post
						$form_id = get_post_meta ( $post_id, 'form_type' )[0];
						$result = $this->ninjaforms->get_submission_data( [ 'submission_id' => $source_id, 'form_id' => $form_id ] );
						$post = get_post( $post_id );
						$post_timestamp = get_post_timestamp( $post );
						$date = new \DateTime( "@$post_timestamp" );
						$date->setTimezone( $wp_timezone );
						$result['time'] = $date->format( $this->time_format );
						$result['date'] = $this->friendly_date( $post, $date->format( $this->date_format ) );
					} else $result = false;
					break;	
			default:
				$result = [ 'name' => __( 'not defined', 'inbound-organizer' ),
							'email' => __( 'not_defined', 'inbound-organizer' ), 
							'subject' => __( 'not_defined', 'inbound-organizer' )
						];
				$post = get_post( $post_id );
				$date = date_i18n ( $this->date_format, get_post_timestamp( $post ) );
				$result['time'] = date_i18n ( $this->time_format, get_post_timestamp( $post ) );
				$result['date'] = $this->friendly_date( $post, $date );
				break;
		}
		return $result;
	}

	/**
	 * Retrieves field data and metadata for a form based on its source and type.
	 *
	 * This function determines the source of the form (e.g., FluentForms, Flamingo, etc.)
	 * and retrieves the field identifiers and mappings for the specified form type.
	 * Metadata such as the option name and form type are also included in the result.
	 * 
	 * @since 1.0.0
	 * 
	 * @param int $post_id The ID of a post of the type inborg_item that represents a form submission in Inbound Organizer.
	 *
	 * @return array {
	 *     An associative array containing the form metadata and field data.
	 *
	 *     @type array $meta {
	 *         Metadata about the form.
	 *
	 *         @type string $option_name The name of the option used for storing the layout (mapping).
	 *         @type string $form_type   The identifier of the form in the form plugin.
	 *     }
	 *     @type array $fields An associative array of field data, where keys are field names.
	 *     @type array $mapping The mapping of three card items items to fields of the form:'name', 'email', 'subject'.
	 * }
	 */
	public function get_field_data( $post_id ) {
		$source = get_post_meta( $post_id, 'source' )[0];
		$source_id = get_post_meta ( $post_id, 'source_id' )[0];
		$form_type = get_post_meta ( $post_id, 'form_type' )[0];
		$result['meta']['option_name'] = 'inbound_organizer_'.$source;
		$result['meta']['form_type'] = $form_type;
		switch ( $source ) {
			case 'fluentform':
				$this->set_fluentforms();
				$result['fields'] = $this->fluentforms->get_fields_inbound( $form_type );
				$result['mapping'] = $this->fluentforms->get_layout( $form_type );
				break;
			case 'flamingo':
				$this->set_flamingo();
				$result['fields'] = $this->flamingo->get_fields_inbound( $form_type );
				$result['mapping'] = $this->flamingo->get_layout( $form_type );
				break;
			case 'cfdb7':
				$this->set_cfdb7();
				$result['fields'] = $this->cfdb7->get_fields_inbound( $form_type );
				$result['mapping'] = $this->cfdb7->get_layout( $form_type );
				break;
			case 'gravityforms':
				$this->set_gravityforms();
				$result['fields'] = $this->gravityforms->get_fields_inbound( $form_type );
				$result['mapping'] = $this->gravityforms->get_layout( $form_type );
				break;
			case 'formidable':
				$this->set_formidable();
				$result['fields'] = $this->formidable->get_fields_inbound( $form_type );
				$result['mapping'] = $this->formidable->get_layout( $form_type );
				break;
			case 'forminator':
				$this->set_forminator();
				$result['fields'] = $this->forminator->get_fields_inbound( $form_type );
				$result['mapping'] = $this->forminator->get_layout( $form_type );
				break;
			case 'ninjaforms':
				$this->set_ninjaforms();
				$result['fields'] = $this->ninjaforms->get_fields_inbound( $form_type );
				$result['mapping'] = $this->ninjaforms->get_layout( $form_type );
				break;
		}
		return $result;
	}
	
	/**
	 * Sets the layout for a form based on its source and type.
	 *
	 * This function determines the source of the form (e.g., FluentForms, Flamingo, etc.)
	 * and delegates the layout configuration to the appropriate handler. 
	 * 
	 * @since    1.0.0
	 * @param array $args {
	 *     Array of arguments required to set the layout.
	 *
	 *     @type int    $post_id   The ID of the post where the form is used.
	 *     @type string $other_key Other keys required for the specific form handler.
	 * }
	 *
	 * @return bool The result of the function update_option as called by the specific form handler.
	 */
	public function set_layout( $args ) {
		$source = get_post_meta( $args['post_id'], 'source' )[0];
		$form_type = get_post_meta ( $args['post_id'], 'form_type' )[0];
		switch ( $source ) {
			case 'fluentform':
				$this->set_fluentforms();
				$result = $this->fluentforms->set_layout( $form_type, $args );
				break;
			case 'flamingo':
				$this->set_flamingo();
				$result = $this->flamingo->set_layout( $form_type, $args );
				break;
			case 'cfdb7':
				$this->set_cfdb7();
				$result = $this->cfdb7->set_layout( $form_type, $args );
				break;
			case 'gravityforms':
				$this->set_gravityforms();
				$result = $this->gravityforms->set_layout( $form_type, $args );
				break;
			case 'formidable':
				$this->set_formidable();
				$result = $this->formidable->set_layout( $form_type, $args );
				break;
			case 'forminator':
				$this->set_forminator();
				$result = $this->forminator->set_layout( $form_type, $args );
				break;
			case 'ninjaforms':
				$this->set_ninjaforms();
				$result = $this->ninjaforms->set_layout( $form_type, $args );
				break;
		}
		return $result;
	}	
}
