( function( $ ) {
	'use strict';

	/**
	 * Attaches JQuery UI Draggable to all divs with the specified class.
	 */
	function makeDraggable() {
		$( '.inbound-organizer-card' ).draggable( {
			containment: '.inbound-organizer-container',
			stack: '.inbound-organizer-container'
			}
		);
	}
	
	/**
	 * Refresh the contents of a column from the server.
	 * 
	 * @param { HTMLElement } column The element that contains the column elements to be refreshed
	 */
	function refreshColumn ( column ) {
		$.post(
			inbound_organizer_ajax_obj.ajax_url, {      			
			_ajax_nonce: inbound_organizer_ajax_obj.nonce,			
			action: 'inbound-organizer_refresh_messages',       
			postData: { 							
						status: column.data( 'status' ),
						count: column.find('.inbound-organizer-card').length
					  }
			}, function ( response ) {
				if ( response.success ) {
					$( column ).find('h2').remove();
					$( column ).find('.inbound-organizer-card').remove();
					$( column ).find('.button-primary').remove();
					$( column ).append ( response.data.html );
					makeDraggable();
				} else {
					console.error( response.data.message );
				}
			} 
		);
	}

	/**
	 * Helper function to process a drag & drop event of a card, 
	 * adding the card to the column in which it was dropped.
	 * 
	 * @param { Event } 	  event 
	 * @param { HTMLElement } card 
	 */
	function handleDropEvent( event, card) {
		var columnTitle = $ ( this ).find( 'h2' ).html(),
			cardTitle = card.draggable.find('h3').html(),
			oldColumn = card.draggable.parent(),
			newColumn = $( this ); 
		$.post(
			inbound_organizer_ajax_obj.ajax_url, {      			
			_ajax_nonce: inbound_organizer_ajax_obj.nonce,			
			action: 'inbound-organizer_update_message_tracking_status',       
			postData: { 							
							status: $(this).data( 'status' ),
							id: card.draggable.data( 'id' )
					  }
			}, function ( response ) {
					if ( response.success ) {
						displayNotification( response.data.message );
						card.draggable.remove();
						refreshColumn( oldColumn);
						refreshColumn( newColumn );
					} else {
						console.error( response.data.message );
					}
			} 
		);
	}

	/**
	 * Display a notification for 7 seconds.
	 * 
	 * @param { String } notification 
	 */
	function displayNotification ( notification ) {
		$( '#inbound-organizer-message' ).html( '<p>' + notification + '</p>' + 
		'<button type="button" class="notice-dismiss">' + 
		'<span class="screen-reader-text">Dismiss this notice.</span></button>' );
		$( '#inbound-organizer-message' ).slideDown(250);
		setTimeout( function () { 
			$( '#inbound-organizer-message' ).slideUp(1000);
			}
			, 7000 
		);
	}
	
	function openCard( cardDetail, display_config_form ) {
		$.post(
			inbound_organizer_ajax_obj.ajax_url, {      			
			_ajax_nonce: inbound_organizer_ajax_obj.nonce,
			action: 'inbound-organizer_render_message_details',
			postData: { 							
						id: cardDetail.data( 'post-id' ),
						config: display_config_form
					  }
			}, 
			function ( response ) {
				if ( response.success ) {
					cardDetail.html( response.data.html );
					$('.ui-dialog').attr('class', cardDetail.data('original-styles') );
					$('.ui-dialog').addClass( $( '#inbound-organizer-card-detail-color-palette').data( 'selected' ) );
					$('.ui-dialog').data( 'original-color', $( '#inbound-organizer-card-detail-color-palette').data( 'selected' ) );
				} else {
					console.error( response.data.message );
				}
			}
		);
		cardDetail.dialog( 'open' );
	}

	/**
	 * Main JQuery function event handlers
	 * 
	 */
	$( document ).ready( function() {
		//Make all messages draggable
		makeDraggable();

		//Event handler for when a message is dropped
		$( '.inbound-organizer-column' ).droppable( {
			drop: handleDropEvent
		} );

		//Definition of the message popup
		var cardDetail = $( '#inbound-organizer-card-detail' ).dialog( {
			autoOpen: false,
			width: 500,
			height: 500,
			show: { effect: 'fadeIn', duration: 500},
			hide: { effect: 'fadeOut', duration: 500}
		  } );
		  cardDetail.data( 'original-styles', $('.ui-dialog').attr('class') );

		//Event handler for clicking on trash icon in overview
		$ ( document ).on( 'click', '.inbound-organizer-card-trash', function ( ) {
			var card = $( this ).parent();
			$.post(
				inbound_organizer_ajax_obj.ajax_url, {      			
				_ajax_nonce: inbound_organizer_ajax_obj.nonce,			
				action: 'inbound-organizer_trash_message',			
				postData: { 							
							id: card.data( 'id' )
						  }
				}, 
				function ( response ) {
					if ( response.success ) { 
								var column = card.parent();
								card.remove();
								refreshColumn ( column );
								displayNotification( response.data.message );
					} else {
						console.error( response.data.message );
					}
				} 
			);
			return false;
		} );

		//Event handler for clicking on trash icon in message popup
		$ ( document ).on( 'click', '#inbound-organizer-card-detail-trash', function ( ) {
			$.post(
				inbound_organizer_ajax_obj.ajax_url, { 
				_ajax_nonce: inbound_organizer_ajax_obj.nonce,
				action: 'inbound-organizer_trash_message',
				postData: {
							id: cardDetail.data( 'post-id' )
						  }
				}, 
				function ( response ) { 
					if ( response.success ) { 
								var column = $('body').find('[data-id="'+cardDetail.data( 'post-id' )+'"]').parent();
								cardDetail.dialog( 'close' );	 
								$('body').find('[data-id="'+cardDetail.data( 'post-id' )+'"]').remove();
								refreshColumn( column );
								displayNotification( response.data.message );
					} else {
						console.error( response.data.message );
					}
				 } 
			);
			return false;
		} );

		//Event handler for clicking on a message in the overview --> show popup
		$( document ).on( 'click', '.inbound-organizer-card', function() {
			cardDetail.data( 'post-id', $( this ).data( 'id' ) );
			cardDetail.data( 'status-id', $( this ).parent().data( 'status' ) );
			openCard( cardDetail, 0 );
		} );

		//Event handler for clicking on a layout config icon in a popup --> refresh popup
		$( document ).on( 'click', '#inbound-organizer-card-detail-config', function() {
			openCard( cardDetail, 1 );
		} );

		//Event handler for closing the message popup
		$ ( document ).on( 'click', '#inbound-organizer-card-detail-close, #modal-background', 
			function () {
				var column = $( 'body' ).find('[data-id="' + cardDetail.data( 'post-id' ) + '"]').parent(); 
				refreshColumn ( column );
				cardDetail.dialog( 'close' );
		} );

		//Event handler to save the notes field of the message in the popup
		$( document ).on( 'submit', '#inbound-organizer-card-detail-save', function () {
			$.post(
				inbound_organizer_ajax_obj.ajax_url, {
				_ajax_nonce: inbound_organizer_ajax_obj.nonce,
				action: 'inbound-organizer_save_message_notes',
				postData: {
							id: $( this ).parent().data( 'post-id' ),
							notes: $( this ).find( 'textarea' ).val()
					  	  }
				},
				function ( response ) {
					if ( ! response.success ) {
						console.error( response.data.message );
					}
				}
			);
			$(this).find( 'textarea' ).prop( 'disabled', true );
			$(this).find( 'button').hide();
			return false;
		} );

		//Event handler to activate notes field in the message popup
		$( document ).on( 'click', '#inbound-organizer-card-detail-save', function() {
			$(this).find( 'textarea' ).prop( 'disabled', false );
			$(this).find( 'textarea' ).focus();
			$(this).find( 'button').show();
		} );

		//Event handler to save the layout configuration in the popup
		//After saving the layout, refresh the card
		$( document ).on( 'submit', '#inbound-organizer-card-layout-save', function () {
			var this2 = $( this );
			$.post(
				inbound_organizer_ajax_obj.ajax_url, {
				_ajax_nonce: inbound_organizer_ajax_obj.nonce,
				action: 'inbound-organizer_save_layout',
				postData: {
							id: $( this ).parent().data( 'post-id' ),
							name: $( this ).find( '[name="name"]' ).val(),
							email: $( this ).find( '[name="email"]' ).val(),
							subject: $( this ).find( '[name="subject"]' ).val(),
					  	  }
				},
				function ( response ) {
					if ( response.success ) {
						//Refresh card page on a successful layout save
						$.post(
							inbound_organizer_ajax_obj.ajax_url, {      			
							_ajax_nonce: inbound_organizer_ajax_obj.nonce,
							action: 'inbound-organizer_render_message_details',
							postData: { 							
										id: this2.parent().data( 'post-id' )
									  }
							}, 
							function ( response ) {
								if ( response.success ) {
									cardDetail.html( response.data.html );
									$('.ui-dialog').attr('class', cardDetail.data('original-styles') );
									$('.ui-dialog').addClass( $( '#inbound-organizer-card-detail-color-palette').data( 'selected' ) );
									$('.ui-dialog').data( 'original-color', $( '#inbound-organizer-card-detail-color-palette').data( 'selected' ) );
								} else {
									console.error( response.data.message );
								}
							}
						);
					} else {
						console.error( response.data.message );
					}
				}
			);
			return false;
		} );


		//Event handler to load more messages
		$( document ).on( 'click', '.inbound-organizer-column .button-primary', function() {
			var this2 = $( this );
			$.post(
					inbound_organizer_ajax_obj.ajax_url, {
					_ajax_nonce: inbound_organizer_ajax_obj.nonce,
					action: 'inbound-organizer_load_more_messages',
					postData: {
								status: $( this ).data( 'status' ),
								page: $( this ).data( 'page' ),
							  } 
					}, 
					function ( response ) {
						if ( response.success ) {
							this2.parent().append( response.data.html );
							this2.remove();
							makeDraggable();
						} else {
							console.error( response.data.message );
						}
					}
				 );
		} );

		//Event handlers for hovering over color palette in card detail view
		$( document ).on( 'mouseenter', '#inbound-organizer-card-detail-color-palette td', 
			function() {
				$( '.ui-dialog' ).removeClass(  $( '.ui-dialog' ).data('original-color') );
				$( '.ui-dialog' ).addClass( $( this ).attr('class'));
			} );
		$( document ).on( 'mouseleave', '#inbound-organizer-card-detail-color-palette td', 
			function() {
				$( '.ui-dialog' ).removeClass( $( this ).attr('class'));
				$( '.ui-dialog' ).addClass(  $( '.ui-dialog' ).data('original-color') );
			} );

		//Event handler for clicking on a color in color palette in card detail view
		$( document ).on( 'click', '#inbound-organizer-card-detail-color-palette td', 
			function() {
				$( '.ui-dialog' ).addClass ( $( this ).attr( 'class' ) );
				$( '.ui-dialog' ).data( 'original-color', $( this ).attr( 'class' ) );
				$.post(
					inbound_organizer_ajax_obj.ajax_url, { 
					_ajax_nonce: inbound_organizer_ajax_obj.nonce,
					action: 'inbound-organizer_update_message_color',
					postData: {
								id: cardDetail.data( 'post-id' ),
								color: $( this ).data( 'index' )
							  } 
					},
					function ( response ) {
						if ( ! response.success ) {
							console.error( response.data.message );
						}
					}
				);
			} );
	} );
} )( jQuery );
