<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with CFDB7 form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from CFDB7 form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer_CFDB7 implements Form_Plugin {
	
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

	/**
	 * Function to add a post to refer to a new CFDB7 submission.
	 * 
	 * This function creates a post of the post type specified in the parent object 
	 * of class Inbound_Organizer_Plugins.
	 * This post is used to track the status of the submission on the Inbound Organizer board.
	 * In the post meta data are stored:
	 * - The fact that this post refers to a submission of CFDB7.
	 * - The id of the form submission in CFDB7.
	 * - The form (type) that was used to create the submission.
	 * - The tracking status on the Inbound Organizer board.
	 * 
	 * If the CFDB7 submission is not found, there function wil return without the creation
	 * of a post.
	 * 
	 * @since 1.0.0
	 * 
	 * @global wpdb $wpdb WordPress database access abstraction object.
	 * 
	 * @param mixed $insert_id The id of the submission in CFDB7. Should be an integer, but it 
	 * 						   could be another type i.e. a string.
	 */
	public function add_message( $insert_id ) {
		global $wpdb;
	
		//The CFDB7 submission is stored in the db7_forms table.
		$table = $wpdb->prefix . "db7_forms";
		
		//Convert the insert_id to integer in case it was provided in another type.
		$insert_id = absint( $insert_id );

		//Prepare and run query to get one row
		//A direct database call is the only method to retrieve this data
		$submission = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table} WHERE form_id = %d", $insert_id ) );

		//This if no submission is found, return without further actions.
		if ( is_null( $submission ) ) return;

		//Create the post.
		$subject = 'CFDB7 submission '.$insert_id;
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									 'post_title' => 'Inbound Organizer CFDB7 submission '.$insert_id,
									 'post_status' => 'publish'
									  ] );
		
		//Store post meta data, provided that the post was created.
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'cfdb7');
			add_post_meta ( $post_id, 'source_id', $insert_id );
			add_post_meta ( $post_id, 'form_type', $submission->form_post_id );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}
		return;
	}
	
	/**
	 * Function to retrieve the fields of a CFDB7 submission.
	 * 
	 * This function retrieves the fields of a CFDB7 submission.
	 * If a mapping to the three items of an Inbound Organizer card exists,
	 * this mapping is used to assign the corresponding field values to these
	 * three card items (name, email, subject).
	 * 
	 * @since 1.0.0
	 * 
	 * @global wpdb $wpdb WordPress database access abstraction object.
	 * 
	 * @param mixed $record_id The id of the submission in CFDB7. Should be an integer, but it 
	 * 						   could be another type i.e. a string.
	 * 
	 * @return array {
	 *     An associative array containing submission data and metadata.
	 *
	 *     @type string $name    The value of the field that is mapped to the card item 'Name'.
	 * 							 If false: the submission was not found or the mapping was not found.
	 * 							 If empty string: there was no field mapped to the card item 'Name'.
	 *     @type string $email   The value of the field that is mapped to the card item 'Email'.
	 * 							 If false: the submission was not found or the mapping was not found.
	 * 							 If empty string: there was no field mapped to the card item 'Email'.
	 *     @type string $subject The value of the field that is mapped to the card item 'Subject'.
	* 							 If false: the submission was not found or the mapping was not found.
	 * 							 If empty string: there was no field mapped to the card item 'Subject'.
	 *     @type string $time    The time of the submission formatted according to the
	 *                           plugin's configured time format.
	 *     @type string $date    The date of the submission formatted according to the
	 *                           plugin's configured date format.
	 *     @type array $fields   Field values of the submission for non-empty fields only. 
	 * 							 The field names are stored as the array keys.
	 * 							 Field values that are arrays, are processed separately.
	 * }
	 */
	public function get_submission_data( $record_id ) {
		global $wpdb;

		//The CFDB7 submission is stored in the db7_forms table.
		$table = $wpdb->prefix . "db7_forms";

		//Initiate the result array.
		$result = ['name' => false, 'email' => false, 'subject' => false, 'fields' => array() ];
		
		//Convert the record_id to integer in case it was provided in another type.
		$record_id = absint( $record_id );
		//A direct database call is the only method to retrieve this data
		$submission = $wpdb->get_row( $wpdb->prepare ( "SELECT * FROM {$table} WHERE form_id = %d", $record_id ) );

		if ( is_null( $submission ) ) return;
		$form_data = unserialize( $submission->form_value );
		$map = array();
		$mapping = get_option( 'inbound_organizer_cfdb7_layouts' );
		if ( $mapping && array_key_exists( $submission->form_post_id, $mapping ) ) {
			$map = $mapping[ $submission->form_post_id ];
			$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
		}
		foreach ( $form_data as $field_key => $field_value ) {
			//Add non-empty fields to the list of fields in the result array.
			if ( !empty( $field_value ) ) {
				//If field_value is an array, we list all values within '[]'
				if ( is_array( $field_value ) ) {
					$array_values = '';
					foreach ( $field_value as $element ) {
						if ( !empty( $array_values ) ) {
							$array_values .= '['.$element.']';	
						} else {
							$array_values = '['.$element.']';
						}
					}
					$field_value = $array_values;
				}
				//Assign the field value to a new array element within fields
				$result['fields'][ $field_key ] = $field_value;
			}

			//If the field_key is in the map, fill the corresponding map_key with the field value, even if empty.
			//This is how a field_value gets assigned to a card item.
			$map_key = array_search( $field_key, $map );
			if ( $map_key ) {
				$result[ $map_key ] = $field_value;
			}
		}
		return $result;
	}	

    /**
     * Retrieve form fields from CFDB7 entries for a specified Form ID.
     *
     * This function queries the database to fetch form field data for a given Contact Form 7 form ID.
     * The form definitions are stored as posts of the post_type `wpcf7_contact_form`.
     * It processes serialized form field values and builds an array of unique field names.
     *
     * @since 1.0.0
     * @global wpdb $wpdb WordPress database access abstraction object.
     * 
     * @param int $form The ID of the form whose fields need to be retrieved.
     * 
     * @return array Associative array of form fields, where the key is the field name 
     *               and the value is the same as the key. Example structure:
     *               [
     *                   'field_name_1' => 'field_name_1',
     *                   'field_name_2' => 'field_name_2',
     *                   ...
     *               ]
     */
    public function get_fields_inbound( $form ) {
		global $wpdb;
		
		// Prepare and execute the database query to fetch entries for the given form ID.
		// A direct database call is the only method to retrieve this data
        $table_name = $wpdb->prefix . 'db7_forms';
		$entries = $wpdb->get_results( 
			$wpdb->prepare( "SELECT * FROM {$table_name} WHERE form_post_id = %d", $form ), 
			ARRAY_A );
		
		$result = array();

		 // Process each entry to extract and store unique form field names.
        foreach ( $entries as $entry ) {
			$fields = unserialize( $entry['form_value'] );
			foreach ( $fields as $field_key => $field_value ) {
				if ( !array_key_exists( $field_key, $result ) ) {
                    $result[ $field_key ] = $field_key;
				}
			}
		}
		return $result;
	}

    /**
     * Retrieve Contact Form 7 form definitions that have associated entries.
     *
     * This function fetches all Contact Form 7 forms (stored as posts of type `wpcf7_contact_form`) 
     * and filters them to include only forms with corresponding entries in the CFDB7 database table.
     *
     * @since 2.0.0
     * @global wpdb $wpdb WordPress database access abstraction object.
     * 
     * @return array List of forms with entries. Each item in the array is an associative array with:
     *               - `id` (int): The ID of the form, which is the post ID.
     *               - `title` (string): The title of the form, which is the post title.
     *               Example:
     *               [
     *                   [
     *                       'id'    => 123,
     *                       'title' => 'Contact Us',
     *                   ],
     *                   [
     *                       'id'    => 124,
     *                       'title' => 'Feedback Form',
     *                   ]
     *               ]
     */
	public function get_forms() {
		global $wpdb;

		$result = array();

		// Retrieve all Contact Form 7 forms (posts of type 'wpcf7_contact_form').
		$forms = get_posts( array( 'post_type' => 'wpcf7_contact_form' ) );
		foreach ( $forms as $form ) {
			// Query the database to check for entries associated with the current form ID.
			// A direct database call is the only method to retrieve this data
			$table_name = $wpdb->prefix . 'db7_forms';
			$entries = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE form_post_id = %d", $form->ID ), 
				ARRAY_A );

            // Include the form only if it has associated entries.
            if ( !empty( $entries ) )  {
			    $result[] = array( 'id'    => $form->ID,
							       'title' => $form->post_title );
            }
		}
		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_cfdb7_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$layouts = get_option( 'inbound_organizer_cfdb7_layouts' );
		
        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $layouts ) ) {
			$layouts = array();
		}

		// Update or add the layout entry for the specified form.
		$layouts[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];
		
		// Save the updated layouts back to the WordPress options table.
		update_option( 'inbound_organizer_cfdb7_layouts', $layouts );
		
		return true;
	}
	
	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_cfdb7_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

        // Retrieve the stored layouts from the WordPress options table.		
		$layouts = get_option( 'inbound_organizer_cfdb7_layouts' );

        // If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $layouts ) && array_key_exists( $form, $layouts ) ) {
			$result = $layouts[ $form ];
		}

		return $result;
	}

}