<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Flamingo form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from Flamingo form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */

class Inbound_Organizer_Flamingo implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

	/**
     * Create a post to reference a new Flamingo form submission.
     *
     * This function generates a new post of the post type specified in the parent object
     * of the `Inbound_Organizer_Plugins` class. The post is used to track the status of the
     * submission on the Inbound Organizer board. Relevant metadata is added to the post to store:
     * - The source of the submission (`flamingo`).
     * - The ID of the Flamingo form submission (`posted_data_hash`).
     * - The type of form used to create the submission (`channel`).
     * - The initial tracking status for the submission (`tracking_status`).
     *
     * If the Flamingo submission is not found, the function exits without creating a post.
     *
     * @since 1.0.0
     * 
     * @param array $args {
     *     An associative array containing the following keys:
     *     @type string $posted_data_hash The unique ID of the Flamingo form submission.
     *     @type string $channel          The ID of the form used for the submission.
     * }
     *
     * @return array The input `$args` array, unchanged.
     */
	public function add_message( $args ) {
		// Create a new post with the specified post type, title, and status.
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									 'post_title' => 'Inbound Organizer Flamingo submission '.$args['posted_data_hash'],
									 'post_status' => 'publish'
									  ] );

        // If the post was successfully created, add metadata to track the submission.
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'flamingo');
			add_post_meta ( $post_id, 'source_id', $args['posted_data_hash'] );
			add_post_meta ( $post_id, 'form_type', $args['channel'] );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}

       // Return the original input arguments.
		return $args;
	}
	
	/**
     * Retrieve submission data based on a unique hash value.
     *
     * This function queries the database for a post of type `flamingo_inbound` with a specific 
     * `_hash` metadata value. It retrieves and processes metadata, terms, and custom fields 
     * associated with the post to return detailed submission data.
     *
     * The function also checks for a form layout configuration for the form ID linked to the submission. 
     * If no layout is found, a default layout is created and saved in the `inbound_organizer_flamingo_layouts` option.
     *
     * @since 1.0.0
     *
     * @param string $hash_value The unique hash value used to identify the Flamingo submission.
     *
     * @return array {
     *     An associative array containing the submission data:
     *
     *     @type string $name      The name of the submitter (default: "not defined").
     *     @type string $email     The email of the submitter (default: "not defined").
     *     @type string $subject   The subject of the submission (default: "not defined").
     *     @type string $date      The formatted date of the submission.
     *     @type string $time      The formatted time of the submission.
     *     @type array  $fields    Custom fields from the submission, where the keys are field names
     *                             and the values are field values (default: empty array).
     * }
     */
	public function get_submission_data( $hash_value ) {		

		$advanced_field_value = function( $post, $field_id ) {
			$value = get_post_meta( $post->ID, $field_id )[0];
			if ( is_array ( $value ) ) {
				$array_values = '';
				foreach ( $value as $element ) {
					if ( !empty( $array_values ) ) {
						$array_values .= '['.$element.']';	
					} else {
						$array_values = '['.$element.']';
					}
				}
				$value = $array_values;
			}
			return $value;
		};

		$result =  ['name' => __( 'not defined', 'inbound-organizer' ),
					'email' => __( 'not_defined', 'inbound-organizer' ), 
					'subject' => __( 'not_defined', 'inbound-organizer' ),
					'date' => date_i18n ( $this->parent->date_format, time() ),
					'time' => date_i18n ( $this->parent->time_format, time() )
					  ];
        
		// Query for posts of type 'flamingo_inbound' matching the provided hash value.
		$query = new \WP_Query( 
			array( 
			'post_type' => 'flamingo_inbound',	
			'meta_key' => '_hash',
			'meta_value' => $hash_value,
			'meta_compare' => '='
			) 
		);
		
		$posts = $query->get_posts();

		if ( $posts ) {
            // Use the first matching post.
			$post = $posts[0];

            // Get the form ID from the terms linked to this post.
			$the_terms = get_the_terms( $post->ID, 'flamingo_inbound_channel' );
			if ( is_array( $the_terms) ){
				$form_id = $the_terms[0]->slug;
			} else {
				return array();
			}

 			// Retrieve or create the layout for the form.
			$layouts = get_option( 'inbound_organizer_flamingo_layouts' );
			if ( $layouts && array_key_exists( $form_id, $layouts ) ) {
				$layout = $layouts[ $form_id ];
			} else {
				$layout = [ 'name' => '_from_name',
							'email' => '_from_email', 
							'subject' => '_subject' ];
				if (! $layouts ) {
					$layouts = array ();
				}
				$layouts[ $form_id ] = $layout;
				update_option( 'inbound_organizer_flamingo_layouts', $layouts );
			}

          	// Retrieve metadata for the submission using the layout fields.
			$result['name'] = array_key_exists( 0, get_post_meta( $post->ID, $layout['name'] ) ) ?  $advanced_field_value( $post, $layout['name'] ) : '-';
			$result['email'] = array_key_exists( 0, get_post_meta( $post->ID, $layout['email'] ) ) ? $advanced_field_value( $post, $layout['email'] ) : '-';
			$result['subject'] = array_key_exists( 0, get_post_meta( $post->ID, $layout['subject'] ) ) ? $advanced_field_value( $post, $layout['subject'] ) : '-';

			// Retrieve custom fields for the submission.
			$fieldnames = get_post_meta( $post->ID, '_fields' )[0];
			foreach ( $fieldnames as $fieldname => $fieldvalue) {
				if ( !empty( get_post_meta( $post->ID, '_field_'.$fieldname ) ) ) {
					$result['fields'][ $fieldname ] =  $advanced_field_value( $post, '_field_'.$fieldname );
				} else {
					$result['fields'][ $fieldname ] = '';
				}
			}
		}

		return $result;
	}
    
    /**
     * Retrieve form fields of Flamingo Inbound Entry fields for a specific form.
     *
     * This function queries Flamingo inbound entries (`flamingo_inbound` post type) for a given 
     * form ID (specified by its slug). The form entries are grouped by Flamingo using terms 
     * (`flamingo_inbound_channel` taxonomy), so the term ID is first retrieved based on the slug.
     * 
     * The function constructs a list of fields discovered in the entries, including default 
     * fields (e.g., subject, name, email).
     *
     * @since 1.0.0
     *
     * @param string $form The slug (title) of the form, used to identify its Flamingo term.
     *
     * @return array {
     *     An associative array of form fields and their mappings.
     *     The array keys are prefixed with `_field_` for dynamically discovered fields 
     *     or use specific default keys for predefined fields.
	 *
     * }
     */
    public function get_fields_inbound( $form ) {
        // The form slug corresponds to a term in the Flamingo taxonomy.
		$term = get_term_by( 'slug', $form, 'flamingo_inbound_channel');

		// Query posts of type 'flamingo_inbound' for entries under the given form term.
		$args = array(
			'post_type' => 'flamingo_inbound',
			'tax_query' => array(
					array(
					'taxonomy' => 'flamingo_inbound_channel',
					'field' => 'slug',
					'terms' => $term->slug,
				),
			),
		);
		$query = new \WP_Query( $args );

		// Default fields provided by Flamingo for Contact Form 7 submissions.
		$result = [ '_subject' => 'Subject', 
					'_from' => 'Name + Email', 
					'_from_name' => 'Name', 
					'_from_email' => 'Email' ];

		// Iterate through the queried posts and retrieve field metadata.
        foreach ( $query->posts as $post ) {
			$meta = get_post_meta( $post->ID );
			$fields = unserialize( $meta['_fields'][0] );

            // Add discovered fields to the result if not already present.
			foreach ( $fields as $field_key => $field_value ) {
				if ( !array_key_exists( '_field_'.$field_key, $result ) ) {
                    $result[ '_field_'.$field_key ] = $field_key;
				}
			}
		}

		return $result;
	}

    /**
     * Retrieve form definitions for Contact Form 7 that have associated submissions in Flamingo.
     *
     * This function fetches the forms created using Contact Form 7 (CF7), stored as posts of the 
     * `wpcf7_contact_form` post type. It only includes forms that have actual entries in Flamingo, 
     * a plugin for storing form submissions. The function queries the `flamingo_inbound` post type 
     * and uses the `flamingo_inbound_channel` taxonomy to ensure that only forms with related submissions 
     * are returned.
     *
     * The returned forms include their IDs and titles.
     *
     * @since    1.0.0
     *
     * @return   array {
     *     An associative array of form definitions, where each entry includes:
     *     
     *     @type integer $id    The ID of the Contact Form 7 form (post ID).
     *     @type string  $title The title of the form (post title).
     * }
     */
	public function get_forms() {
		$result = array();

        // Retrieve all CF7 forms.
		$forms = get_posts( array( 'post_type' => 'wpcf7_contact_form' ) );
		foreach ( $forms as $form ) {
            
            // Fetch the term (category) associated with this form in Flamingo.
            $term = get_term_by( 'name', $form->post_title, 'flamingo_inbound_channel');

			if ( $term ) {
				// Query entries in the 'flamingo_inbound' post type, filtered by the term slug.
				$args = array(
					'post_type' => 'flamingo_inbound',
					'tax_query' => array(
							array(
							'taxonomy' => 'flamingo_inbound_channel',
							'field' => 'slug',
							'terms' => $term->slug,
						),
					),
				);
				$query = new \WP_Query( $args );

                // Only add the form to the result if there are related submissions.
				if ( $query->have_posts() )  {
					$result[] = array( 
						'id'    => $form->ID,
						'title' => $form->post_title 
					);
				}
			}
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_flamingo_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$layouts = get_option( 'inbound_organizer_flamingo_layouts' );

        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $layouts ) ) {
			$layouts = array();
		}
		
		// Update or add the layout entry for the specified form.
		$layouts[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.
		update_option( 'inbound_organizer_flamingo_layouts', $layouts );

		return true;
	}
	
	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_flamingo_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

        // Retrieve the stored layouts from the WordPress options table.
		$layouts = get_option( 'inbound_organizer_flamingo_layouts' );
        
		// If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $layouts ) && array_key_exists( $form, $layouts ) ) {
			$result = $layouts[ $form ];
		}

		return $result;
	}
}