<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Fluent Form form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from Fluent Form form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer_FluentForms implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}
	
	/**
     * Add a Fluent Forms submission to the Inbound Organizer.
     *
     * This function is triggered by the `fluentform/submission_inserted` action hook. It creates a new post in the Inbound Organizer 
     * to track the submission from Fluent Forms. The post includes relevant metadata such as the source of the submission, 
     * the form ID, and a tracking status. The created post is of a custom post type defined in the parent object of the class.
     *
     * @since 1.0.0
     *
     * @param array $args An associative array containing the following:
     *     @type integer $submissionId The unique ID of the submission in Fluent Forms.
     *     @type array   $formData     The submission data from the Fluent Form.
     *     @type object  $form         The form object associated with this submission.
     * 
     * @return void
     */
	public function add_message( $args ) {
      // Insert a new post for this Fluent Forms submission.
	  $post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									'post_title' => 'Inbound Organizer FluentForms submission '.$args['submissionId'],
									'post_status' => 'publish'
		] );
      	
		// If the post was successfully created, add relevant metadata.
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'fluentform');
			add_post_meta ( $post_id, 'source_id', $args['submissionId'] );
			add_post_meta ( $post_id, 'form_type', $args['form']->id );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}

		return;
	}

	/**
     * Retrieve the submission data for a specific submission ID from Fluent Form.
     *
     * This function queries the Fluent Forms entry details based on the provided submission ID. It returns an array containing the submission 
     * data, including the name, email, subject, and any other fields associated with the form submission. The function also uses a mapping 
     * of form fields (if available) to map the submitted field values to more readable names.
     * 
     * The function first checks if there are entry details associated with the submission. If a mapping exists for the form ID, the function 
     * uses that mapping to populate the result array. The result includes the form's specific fields and their corresponding values.
     *
     * @since 1.0.0
	 * 
	 * @global wpdb $wpdb WordPress database access class used for interacting with the database.
     *
     * @param int $id The unique ID of the Fluent Form submission.
     *
     * @return array The submission data, with the following keys:
     *     - 'name' => The name value from the submission (if available).
     *     - 'email' => The email value from the submission (if available).
     *     - 'subject' => The subject value from the submission (if available).
     *     - 'fields' => An associative array of all other fields, with field names as keys and their values.
     * 
     *     The fields are dynamically mapped based on the form layout settings, and defaults to empty values if no mapping is found.
     */
	public function get_submission_data( $id ) {
		global $wpdb;

		// Initialize the result with default values
		$result = ['name' => false, 'email' => false, 'subject' => false];
		$map = array();

		// Prepare and execute the query to retrieve the entry details for the given submission ID
        // A direct database call is the only method to retrieve this data
        $table_name = $wpdb->prefix.'fluentform_entry_details';
        $entry_details = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE submission_id = %d LIMIT 200", $id ), 
            ARRAY_A );
        // If no entry details are found, return the default result
		if ( empty ( $entry_details ) ) {
			return $result;
		}

		// Retrieve the field mapping from the plugin settings
        $mapping = get_option( 'inbound_organizer_fluentform_layouts' );

        // If a mapping exists for this form, use it to initialize the result fields
		if ( $mapping && array_key_exists( $entry_details[0]['form_id'], $mapping ) ) {
			$map = $mapping[ $entry_details[0]['form_id'] ];
			$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
		};
		
        // Iterate over the entry details and populate the result array with field values
		foreach ( $entry_details as $entry_detail ) {
			if ( empty( $entry_detail['sub_field_name'] ) ) { 
				$field_key = $entry_detail['field_name']; 
			} else {
				$field_key = $entry_detail['field_name'].'_'.$entry_detail['sub_field_name'];
			}

            // If the field is in the map, assign the value to the corresponding result field
			$map_key = array_search( $field_key, $map );
			if ( $map_key ) {
				$result[ $map_key ] = $entry_detail['field_value'];
			}

			// Add the field value to the 'fields' array in the result
			$result['fields'][ $field_key ] = $entry_detail[ 'field_value' ];
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_fluentform_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$mapping = get_option( 'inbound_organizer_fluentform_layouts' );

        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $mapping ) ) {
			$mapping = array();
		}
		
		// Update or add the layout entry for the specified form.
		$mapping[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.
		update_option( 'inbound_organizer_fluentform_layouts', $mapping );

		return true;
	}

	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_fluentform_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

        // Retrieve the stored layouts from the WordPress options table.		
		$mapping = get_option( 'inbound_organizer_fluentform_layouts' );

		// If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $mapping ) && array_key_exists( $form, $mapping ) ) {
			$result = $mapping[ $form ];
		}

		return $result;
	}

    /**
     * Retrieve form fields for a specific Fluent Form by its form ID.
     * 
     * This function queries the `fluentform_entry_details` table to retrieve the first 100 entries 
     * for the specified form. It distills the form fields from these entries, considering both the 
     * main field names and any sub-field names (if applicable). The result is an associative array 
     * with technical field names as keys and user-friendly field names as values.
     * 
     * The function supports fields with nested structures, where sub-fields are appended to the main 
     * field name with a `_` delimiter. This is done to ensure that sub-fields are correctly identified 
     * and labeled.
     * 
     * @since 1.0.0
     * 
     * @global wpdb $wpdb WordPress database access class, used to interact with the WordPress database.
     * 
     * @param int $form The unique form ID to retrieve fields for.
     *
     * @return array An associative array where the keys are the technical field names (including 
     *               any sub-field names) and the values are the corresponding user-friendly field names.
     */
    public function get_fields_inbound( $form ) {
		global $wpdb;

	    // Retrieve the first 100 entries for the specified form ID from the entry details table
        // A direct database call is the only method to retrieve this data
        $table_name = $wpdb->prefix.'fluentform_entry_details';
        $entry_details = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE form_id = %d LIMIT 100", $form ), 
            ARRAY_A );

		$result = array();

        // Iterate through each entry to extract field names
		foreach ( $entry_details as $entry_detail ) {
            // Check if the entry has a sub-field; if not, use the main field name
			if ( empty( $entry_detail['sub_field_name'] ) ) { 
				$field_key = $entry_detail['field_name'];
				$field_name = $entry_detail['field_name'];
			} else {
				$field_key = $entry_detail['field_name'].'_'.$entry_detail['sub_field_name'];
				$field_name = $entry_detail['sub_field_name'].' ('.$entry_detail['field_name'].')';
			}

			// If this field has not already been added to the result, add it with its user-friendly name
			if ( !array_key_exists( $field_key, $result ) ) {
                    $result[ $field_key ] = $field_name;
			}
		}

		return $result;
	}

    /**
     * Retrieve form definitions from Fluent Forms that have entries.
     * 
     * This function queries the `fluentform_forms` table to retrieve all available form definitions.
     * It then checks the `fluentform_submissions` table to see if any entries exist for each form.
     * Only forms with related entries are returned in the result. Each form’s ID and title are included 
     * in the result array.
     * 
     * @since    1.0.0
     * @global   \wpdb $wpdb The WordPress database object, used to interact with the WordPress database.
     * 
     * @return   array An indexed array of form definitions that have entries, with each form represented as 
     *                 an associative array containing:
     *                 - `id`: The unique ID of the form.
     *                 - `title`: The title of the form.
     *                 
     *                 Example return structure:
     *                 [
     *                     [ 'id' => 1, 'title' => 'Contact Form' ],
     *                     [ 'id' => 2, 'title' => 'Newsletter Signup' ]
     *                 ]
     */
	public function get_forms() {
		global $wpdb;

		$result = array();

		// Retrieve all form definitions from the fluentform_forms table
        // A direct database call is the only method to retrieve this data
        $forms = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}fluentform_forms", ARRAY_A );

		foreach ( $forms as $form ) {
            // Check if the form has any submissions by querying the fluentform_submissions table
            // A direct database call is the only method to retrieve this data
            $table_name = $wpdb->prefix.'fluentform_submissions';
        	$entry_details = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE form_id = %d", $form['id'] ), ARRAY_A );
		
            // Only add forms that have associated entries to the result
            if ( !empty( $entry_details ) )  {
			    $result[] = array( 'id'    => $form['id'],
							       'title' => $form['title'] );
            }
		}

		return $result;
	}

}