<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Formidable form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from CFDB7 form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer_Formidable implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

	/**
     * Add a Formidable submission to the Inbound Organizer.
     *
     * This function is triggered by the `frm_after_create_entry` action hook. It creates a new post in the Inbound Organizer 
     * to track the submission from Formidable. The post includes relevant metadata such as the source of the submission, 
     * the form ID, and a tracking status. The created post is of a custom post type defined in the parent object of the class.
     *
     * @since 1.0.0
     *
     * @param array $args An associative array containing the following:
     *     @type integer $id 		The unique ID of the submission in Formidable.
     *     @type object  $form_id   The form object associated with this submission.
     * 
     * @return void
     */
	public function add_message( $args ) {
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									'post_title' => 'Inbound Organizer Formidable submission '.$args['id'],
									'post_status' => 'publish'
		] );

		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'formidable');
			add_post_meta ( $post_id, 'source_id', $args['id'] );
			add_post_meta ( $post_id, 'form_type', $args['form_id'] );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}
		return;
	}

	/**
     * Retrieve the submission data for a specific Formidable Form submission.
     * 
     * This function retrieves the submission data for a given submission ID from the Formidable Forms plugin.
     * It first checks if the Formidable plugin is active, then it fetches the associated form and fields.
     * It also checks for any custom mappings in the layout settings and returns the submission data in a structured format.
     * The function handles different field types, including array-based fields (e.g., checkboxes or multi-select fields).
     * 
     * @since    1.0.0
     * @global   \FrmEntry $entry The Formidable entry object, which contains the submission data.
     * @global   \FrmForm $form The Formidable form object, which defines the form structure.
     * @global   \FrmField[] $fields An array of form fields that belong to the form.
     * 
     * @param int $submission_id The ID of the submission to retrieve data for.
     * 
     * @return array {
     *     Returns an associative array containing the following fields:
     *     - `name`: The value of the name field (if available).
     *     - `email`: The value of the email field (if available).
     *     - `subject`: The value of the subject field (if available).
     *     - `fields`: An array of all field values for the submission, indexed by field name.
     *     The array might look like:
     *     [
     *         'name' => 'John Doe',
     *         'email' => 'johndoe@example.com',
     *         'subject' => 'Inquiry',
     *         'fields' => [
     *             'message' => 'I have a question about your services.',
     *             'interest' => 'Yes, I am interested in your products.'
     *         ]
     *     ]
     * }
     */
	public function get_submission_data( $submission_id ) {
		$result = ['name' => false, 'email' => false, 'subject' => false];

        // Ensure Formidable Forms is active and available
		if ( class_exists('FrmEntry') ) {

			// Get the submission (entry) by ID
			$entry = \FrmEntry::getOne( $submission_id, true );
			if ( !$entry ) return $result;
			
			// Get the associated form object for this submission
			$form = \FrmForm::getOne( $entry->form_id) ;
			if ( !$form ) return $result;

			// Get all the fields associated with the form
    		$fields = \FrmField::getAll( ['fi.form_id' => $form->id] );

			// Retrieve layout mappings from the options table
			$map = array();
			$mapping = get_option( 'inbound_organizer_formidable_layouts' );
			if ( $mapping && array_key_exists( $form->id, $mapping ) ) {
				$map = $mapping[ $form->id ];
				$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
			};

			// Loop through each field and extract its submitted value
			foreach ($fields as $field) {
				$field_name = $field->name;
				$field_id = $field->id;
				
				// Retrieve the value for this field from the entry
				$field_value = isset($entry->metas[$field_id]) ? $entry->metas[$field_id] : '';
				
				// Handle array or complex values (e.g., checkboxes, multi-select fields)
				if ( is_array($field_value) ) {
					$field_value = implode(', ', $field_value);
				}

                // If the field is mapped, assign its value to the corresponding result key
				$map_key = array_search( $field_id, $map );
				if ( $map_key ) {
					$result[ $map_key ] = $field_value;
				}

                // Add field value to the 'fields' key for all fields, even unmapped ones
				if ( !empty( $field_value) ) {
					$result['fields'][ $field_name ] = $field_value;
				}
			}
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_formidable_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$mapping = get_option( 'inbound_organizer_formidable_layouts' );

        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $mapping ) ) {
			$mapping = array();
		}
		
		// Update or add the layout entry for the specified form.
		$mapping[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.
		update_option( 'inbound_organizer_formidable_layouts', $mapping );
		return true;
	}

	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_formidable_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

        // Retrieve the stored layouts from the WordPress options table.		
		$mapping = get_option( 'inbound_organizer_formidable_layouts' );

        // If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $mapping ) && array_key_exists( $form, $mapping ) ) {
			$result = $mapping[ $form ];
		}

		return $result;
	}

	/**
     * Get all form fields for a specific Formidable Form ID.
     * 
     * This function retrieves the fields of a Formidable form by its ID. It checks if the Formidable Forms plugin is active,
     * retrieves the form and its associated fields, and returns an associative array with field IDs as keys and field names as values.
     * 
     * @since    1.0.0
     * 
     * @param int $form_id The ID of the Formidable form for which the fields should be retrieved.
     * 
     * @return array Returns an associative array where each key is a field ID and the corresponding value is the field name.
     *               For example:
     *               [
     *                   1 => 'Name',
     *                   2 => 'Email',
     *                   3 => 'Message'
     *               ]
     *               If no fields are found or the Formidable plugin is not active, an empty array is returned.
     */
    public function get_fields_inbound( $form_id ) {
		$result = array();

		// Check if the Formidable Forms plugin is active and the FrmEntry class is available
		if ( ! class_exists('FrmEntry') ) {
			return $result; // Return empty array if Formidable Forms is not active
		}

        // Get the form object by ID
		$form = \FrmForm::getOne( $form_id );
		if ( !$form ) {
			return $result; // Return empty array if the form is not found
		}

        // Retrieve all fields associated with the form
		$fields = \FrmField::getAll(['fi.form_id' => $form->id]);

        // Loop through each field and store its ID and name in the result array
		foreach ( $fields as $field ) {
			$field_name = $field->name;
			$field_id = $field->id;
			$result[ $field_id ] = $field_name;	// Store field ID as key and field name as value
		 }

		return $result;  // Return the associative array of field IDs and names
	}

    /**
	 * Get all Formidable forms that have entries.
	 * 
	 * This function retrieves form definitions using Formidable's API and includes only those forms
	 * that have at least one associated entry.
	 * 
	 * @since  1.0.0
	 * 
	 * @return array An array of forms that have entries. Each element is an associative array with:
	 *               - `id` (int): The form ID.
	 *               - `title` (string): The form's name (title).
	 * 
	 *               Example return value:
	 *               [
	 *                   ['id' => 1, 'title' => 'Contact Form'],
	 *                   ['id' => 2, 'title' => 'Survey Form']
	 *               ]
	 *               If no forms with entries are found, an empty array is returned.
	 */
	public function get_forms() {
		$result = array();

		// Check if the Formidable Forms plugin is active and the FrmForm class is available
		if ( ! class_exists('FrmForm') ) {
			return $result; // Return empty array if Formidable Forms is not active
		}

		// Use the Formidable API to get all forms
		$forms = \FrmForm::getAll();

		// Loop through each form
		foreach ( $forms as $form ) {
			// Check if the form has any associated entries
			$entry_count = \FrmEntry::get_count( array( 'form_id' => $form->id ) );
			if ( $entry_count > 0 ) {
				$result[] = array(
					'id'    => $form->id,
					'title' => $form->name,
				);
			}
		}

		return $result;
	}
}