<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Forminator form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from Forminator form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer_Forminator implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

	/**
     * Add a Forminator submission to the Inbound Organizer.
     *
     * This function is triggered by the `forminator_form_after_save_entry` action hook. It creates a new post in the Inbound Organizer 
     * to track the submission from Forminator. The post includes relevant metadata such as the source of the submission, 
     * the form ID, and a tracking status. The created post is of a custom post type defined in the parent object of the class.
     *
     * @since 1.0.0
     *
     * @param array $args An associative array containing the following:
     *     @type integer $id 		The unique ID of the submission in Forminator.
     *     @type object  $form_id   The form object associated with this submission.
     * 
     * @return void
     */
	public function add_message( $args ) {
		//Create the post.
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									'post_title' => 'Inbound Organizer Forminator submission '.$args['id'],
									'post_status' => 'publish'
		] );
	
		//Store post meta data, provided that the post was created.
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'forminator');
			add_post_meta ( $post_id, 'source_id', $args['id'] );
			add_post_meta ( $post_id, 'form_type', $args['form_id'] );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}
		return;
	}

	/**
     * Get submission data for a specific entry in Forminator.
     * 
     * This function retrieves the submission data for a specific entry in Forminator. It first fetches the form entry
     * object using the Forminator API. Then, it extracts the meta data (field values) from the entry and maps them to 
     * predefined labels or keys based on the layout configuration.
     * 
     * The result includes the submission fields along with a mapping to key data such as 'name', 'email', and 'subject'.
     * The layout configuration, which is stored in WordPress options, determines how field values are mapped to these keys.
     * 
     * @since  1.0.0
     * 
     * @param array $args {
     *     The arguments for retrieving submission data.
     *     
     *     @type int $form_id   The ID of the form.
     *     @type int $entry_id  The ID of the specific submission (entry).
     * }
     * 
     * @return array The submission data, including field values and mapped key data.
     * 
     * Example return:
     * [
     *     'name' => 'John Doe',
     *     'email' => 'john@example.com',
     *     'subject' => 'Inquiry',
     *     'fields' => [
     *         'name' => 'John Doe',
     *         'email' => 'john@example.com',
     *         'message' => 'Hello, I have a question.'
     *     ]
     * ]
     * If no entry is found or the Forminator API class doesn't exist, returns default values:
     * [
     *     'name' => false,
     *     'email' => false,
     *     'subject' => false,
     *     'fields' => []
     * ]
     */
	public function get_submission_data( $args ) {

		$result = ['name' => false, 'email' => false, 'subject' => false];

		// Check if the Forminator API class exists before proceeding.
		if ( class_exists( 'Forminator_API' ) ) {
            // Retrieve the form entry object using the form ID and entry ID from the arguments.
			$entry = \Forminator_API::get_entry( $args['form_id'], $args['entry_id'] );

			// If no entry is found, return the default result.
			if ( ! $entry ) {
				return $result;
			}

            // Extract meta data (field values) from the entry object.
    		$fields = $entry->meta_data;

            // Get the layout mapping from WordPress options (used to map field values to key labels).
			$map = array();
			$mapping = get_option( 'inbound_organizer_forminator_layouts' );

            // If a mapping exists for the current form, apply it to the result.
			if ( $mapping && array_key_exists( $args['form_id'], $mapping ) ) {
				$map = $mapping[ $args['form_id'] ];
				$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
			};

			// Initialize the 'fields' array to store individual field values.
			$result['fields'] = array();

			// Loop through the fields and populate the result array.
			foreach ( $fields as $field_key => $field_value ) {
                // Store the field value in the 'fields' array with its field key.
				$result['fields'][ $field_key ] = $field_value['value'];

                // If the field is mapped, update the corresponding field in the result.
				$map_key = array_search( $field_key, $map );
				if ( $map_key ) {
					$result[ $map_key ] = $field_value['value'];
				}
			}
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_forminator_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$mapping = get_option( 'inbound_organizer_forminator_layouts' );

        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $mapping ) ) {
			$mapping = array();
		}

		// Update or add the layout entry for the specified form.
		$mapping[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.
		update_option( 'inbound_organizer_forminator_layouts', $mapping );

		return true;
	}

	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_forminator_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

        // Retrieve the stored layouts from the WordPress options table.		
		$mapping = get_option( 'inbound_organizer_forminator_layouts' );

		// If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $mapping ) && array_key_exists( $form, $mapping ) ) {
			$result = $mapping[ $form ];
		}

		return $result;
	}

    /**
     * Get all form fields for a specific Forminator form ID.
     * 
     * This function retrieves the fields of a Forminator form by its ID. It checks if the Forminator plugin is active,
     * retrieves the form and its associated fields, and returns an associative array with field IDs (slugs) as keys and values.
     * 
     * @since    1.0.0
     * 
     * @param int $form_id The ID of the Formidable form for which the fields should be retrieved.
     * 
     * @return array Returns an associative array with field IDs (slugs) as keys and values.
     */
    public function get_fields_inbound( $form ) {
		$result = array();

		// Check if the Forminator plugin is active and the Forminator_API class is available
		if ( class_exists( 'Forminator_API' ) ) {
        	// Get the form object by ID
			$form = \Forminator_API::get_form( $form );
			if ( is_wp_error( $form ) ) {
				return $result; // Return empty array if the form is not found
			}

			// Loop through each field and store its slug in the result array as key and value
			foreach ( $form->fields as $field ) {
				$result[ $field->slug ] = $field->slug;
			}
		}

		return $result;
	}

	
	/**
	 * Get all Forminator form definitions with existing submissions.
	 * 
	 * This function retrieves all Forminator forms using the `Forminator_API` and filters out forms
	 * that do not have any submissions. The forms are returned as an associative array containing 
	 * their IDs and titles.
	 * 
	 * @since  1.0.0
	 * 
	 * @return array An associative array of form definitions with existing submissions.
	 *               Example:
	 *               [
	 *                   [
	 *                       'id'    => 1,
	 *                       'title' => 'Contact Form',
	 *                   ],
	 *                   [
	 *                       'id'    => 2,
	 *                       'title' => 'Feedback Form',
	 *                   ]
	 *               ]
	 */
	public function get_forms() {
		$result = array();

		// Check if Forminator_API is available.
		if ( ! class_exists( 'Forminator_API' ) ) {
			return $result;
		}

		// Retrieve all forms using the Forminator API.
		$forms = \Forminator_API::get_forms();

		// Loop through each form to check if it has submissions.
		foreach ( $forms as $form ) {
			// Retrieve the submission count for the form.
			$submission_count = \Forminator_API::count_submissions( $form->id );

			// Include the form in the result only if it has submissions.
			if ( $submission_count > 0 ) {
				$result[] = array(
					'id'    => $form->id,
					'title' => $form->name,
				);
			}
		}

		return $result;
	}
}