<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Gravity Forms submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from Gravity Forms submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
 class Inbound_Organizer_GravityForms implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

	/**
     * Add a Gravity Forms submission to the Inbound Organizer.
     *
     * This function is triggered by the `gform_after_submission` action hook. It creates a new post in the Inbound Organizer 
     * to track the submission from Gravity Forms. The post includes relevant metadata such as the source of the submission, 
     * the form ID, and a tracking status. The created post is of a custom post type defined in the parent object of the class.
     *
     * @since 1.0.0
     *
     * @param array $args An associative array containing the following:
     *     @type integer $id 		The unique ID of the submission in Gravity Forms.
     *     @type object  $form_id   The form object associated with this submission.
     * 
     * @return void
     */
	public function add_message( $args ) {
		//Create the post.
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									'post_title' => 'Inbound Organizer GravityForms submission '.$args['id'],
									'post_status' => 'publish'
		] );

		//Store post meta data, provided that the post was created.
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'gravityforms');
			add_post_meta ( $post_id, 'source_id', $args['id'] );
			add_post_meta ( $post_id, 'form_type', $args['form_id'] );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}
		return;
	}

	/**
	 * Retrieve submission data for a given Gravity Forms submission ID.
	 *
	 * This function retrieves the submission (entry) by its ID using the Gravity Forms API.
	 * It maps form fields to their respective values, and optionally applies a user-defined layout
	 * mapping (from `inbound_organizer_gravityforms_layouts` option) to extract specific meta fields
	 * like `name`, `email`, and `subject`.
	 *
	 * @since 1.0.0
	 *
	 * @param int $submission_id The unique ID of the Gravity Forms submission (entry).
	 *
	 * @return array {
	 *     An associative array containing the submission data:
	 * 
	 *     @type string|false $name    The name field value, or false if not found/mapped.
	 *     @type string|false $email   The email field value, or false if not found/mapped.
	 *     @type string|false $subject The subject field value, or false if not found/mapped.
	 *     @type array        $fields  Key-value pairs of field labels and their corresponding values.
	 * }
	 */
	public function get_submission_data( $submission_id ) {
		$result = ['name' => false, 'email' => false, 'subject' => false];
    	// Ensure Gravity Forms API class exists
		if ( class_exists('GFAPI') ) {
        	// Retrieve the submission (entry) by ID
			$entry = \GFAPI::get_entry( $submission_id );
			if ( is_wp_error( $entry ) ) {
				return $result;
			} 
		
			// Retrieve the associated form object
			$form = \GFAPI::get_form( $entry['form_id'] );
			if ( !$form ) {
				return $result;
			}

			// Retrieve custom layout mappings, if available
			$map = array();
			$mapping = get_option( 'inbound_organizer_gravityforms_layouts' );
			if ( $mapping && array_key_exists( $entry['form_id'], $mapping ) ) {
				$map = $mapping[ $entry['form_id'] ];
				$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
			};

			// Iterate through the form fields to extract field data
			foreach ( $form['fields'] as $field ) {
      		    // Check for fields with subfields (e.g., Name, Address)
				if ( is_array( $field->inputs ) && !empty( $field->inputs ) ) {
					// Iterate through subfields
					foreach ( $field->inputs as $input ) {
						$subfield_label = $input['label'];
						$input_id = $input['id'];
						$subfield_value = rgar( $entry, $input_id, '-' );
						$field_key = $input['id'];
						
						// Map subfield values to meta keys if defined in the layout
						$map_key = array_search( $field_key, $map );
						if ( $map_key ) {
							$result[ $map_key ] = wp_strip_all_tags(  \GFCommon::get_lead_field_display($field, $subfield_value, $entry['currency']) );
						}

						// Add subfield values to the fields array if not empty
						if ( !empty( $subfield_value ) ) {
							//Add it to the array
							$result['fields'][ $subfield_label ] = wp_strip_all_tags( \GFCommon::get_lead_field_display($field, $subfield_value, $entry['currency']) );
						}
					}
				} else {
					// Handle standard fields without subfields
					$field_label = $field->label;
					$field_id = $field->id;
					
					// Get the value submitted for this field
					$field_value = rgar( $entry, $field_id, '-' );

					 // Map field values to meta keys if defined in the layout
					$map_key = array_search( $field_id, $map );
					if ( $map_key ) {
						$result[ $map_key ] = wp_strip_all_tags( \GFCommon::get_lead_field_display($field, $field_value, $entry['currency']) );
					}

					 // Add field values to the fields array if not empty
					if ( !empty( $field_value ) ) {	
						//Add it to the array
						$result['fields'][ $field_label ] = wp_strip_all_tags( \GFCommon::get_lead_field_display($field, $field_value, $entry['currency']) );
					}
				}
			}
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_gravityforms_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$mapping = get_option( 'inbound_organizer_gravityforms_layouts' );

        // If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $mapping ) ) {
			$mapping = array();
		}

		// Update or add the layout entry for the specified form.
		$mapping[ $form ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.
		$result = update_option( 'inbound_organizer_gravityforms_layouts', $mapping );
		
		return true;
	}

	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_gravityforms_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];
		
     	// Retrieve the stored layouts from the WordPress options table.	
	 	$mapping = get_option( 'inbound_organizer_gravityforms_layouts' );

		// If layouts exist and the form ID is found, return its layout configuration.
	  	if ( is_array( $mapping ) && array_key_exists( $form, $mapping ) ) {
			$result = $mapping[ $form ];
		}

		return $result;
	}

    /**
	 * Retrieve form fields for a specific Gravity Forms Form ID.
	 *
	 * This function extracts all fields for the specified form, including subfields for compound fields 
	 * (e.g., Name or Address). The fields are returned as an associative array where the keys are the 
	 * technical field IDs and the values are user-friendly field names.
	 *
	 * @since 1.0.0
	 *
	 * @param int $form The ID of the Gravity Forms form.
	 *
	 * @return array {
	 *     An associative array of form fields.
	 *
	 *     @type string $field_id The technical field ID (including subfield IDs for compound fields).
	 *     @type string $field_label The user-friendly field label, or a combination of parent field and subfield labels.
	 * }
	 */
    public function get_fields_inbound( $form ) {
		$result = array();
		
		// Check if the Gravity Forms API is available
		if ( class_exists('GFAPI') ) {
			// Retrieve the form object by ID
			$form = \GFAPI::get_form( $form );	
			if ( !$form ) {
				return $result;
			}

			// Iterate through the form fields
			foreach ( $form['fields'] as $field ) {
				// Check if the field has subfields (e.g., Name, Address)
				if ( is_array( $field->inputs ) && !empty( $field->inputs ) ) {
					// Iterate through subfields
					foreach ( $field->inputs as $input ) {
						$subfield_label = $input['label'];

						// Add the subfield to the result array with a combined label
						$result[ $input['id'] ] = $field->label . ' - ' . $subfield_label ;	
					}
				} else {
					// For standard fields without subfields
					$result[ $field->id ] = $field->label;
				}
			}
		}

		return $result;
	}

    /**
	 * Get Gravity Forms definitions that have submissions.
	 * 
	 * Retrieves a list of Gravity Forms that have at least one associated entry.
	 * This function uses the Gravity Forms API (GFAPI) to fetch forms and their submissions.
	 * 
	 * @since  1.0.0
	 * 
	 * @return array {
	 *     List of forms with submissions.
	 *     @type integer $id    The form ID.
	 *     @type string  $title The form title.
	 * }
	 */
	public function get_forms() {
		$result = array();

		// Use the Gravity Forms API to get all forms
		$forms = \GFAPI::get_forms();
		if ( empty( $forms ) ) {
			return $result; // Return an empty array if no forms are found
		}

		// Iterate through each form
		foreach ( $forms as $form ) {
			// Check if the form has any entries
			$entry_count = \GFAPI::count_entries( $form['id'] );
			if ( $entry_count > 0 ) {
				$result[] = array(
					'id'    => $form['id'],
					'title' => $form['title'],
				);
			}
		}

		return $result;
	}
}