<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Handles functionality for interfacing with Ninja Forms form submissions.
 *
 * This class implements the `Form_Plugin` interface, as defined in the `Inbound_Organizer_Plugins` class, 
 * to manage and organize data from Ninja Forms form submissions.
 *
 * @link       https://plugins.internetmanagers.nl
 * @since      1.0.0
 * 
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/admin/supported-plugins
 * 
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
 class Inbound_Organizer_NinjaForms implements Form_Plugin {
	/**
	 * Reference to the parent object of class Inbound_Organizer_Plugins 
	 * that instantiates this object.
	 *
	 * @since    1.0.0
	 * @var      Inbound_Organizer_Plugins $parent Object reference.
	 */
	private $parent;
	
	/**
	 * Class constructor for initializing the plugin integration object.
	 *
	 * This constructor sets up the plugins reference to the initiating parent object.
	 *
	 * @since 1.0.0
	 *
	 * @param Inbound_Organizer_Plugins $parent Reference to the instantiating parent object.
	 */
	public function __construct( $parent ) {
		$this->parent = $parent;
	}

    
	/**
     * Add a Ninja Forms submission to the Inbound Organizer.
     *
     * This function is triggered by the `ninja_forms_after_submission` action hook. It creates a new post in the Inbound Organizer 
     * to track the submission from Ninja Forms. The post includes relevant metadata such as the source of the submission, 
     * the form ID, and a tracking status. The created post is of a custom post type defined in the parent object of the class.
     *
     * @since 1.0.0
     *
     * @param array $args An associative array containing the following:
     *     @type integer $id 		The unique ID of the submission in Ninja Forms.
     *     @type object  $form_id   The form object associated with this submission.
     * 
     * @return void
     */
	public function add_message( $args ) {
		$post_id = wp_insert_post( [ 'post_type' => $this->parent->post_type,
									'post_title' => 'Inbound Organizer Ninja Forms submission '.$args['id'],
									'post_status' => 'publish'
		] );
		if ( $post_id > 0 ) {
			add_post_meta ( $post_id, 'source', 'ninjaforms');
			add_post_meta ( $post_id, 'source_id', $args['id'] );
			add_post_meta ( $post_id, 'form_type', $args['form_id'] );
			add_post_meta ( $post_id, 'tracking_status', '1' );
		}
		return;
	}
	
	/**
	 * Retrieve submission data from a Ninja Forms submission.
	 * 
	 * This function extracts submission data for a given submission ID and form ID, maps field values to user-defined keys,
	 * and returns an array with field values and metadata (e.g., name, email, subject). The function also ensures
	 * compatibility with field mappings stored in options.
	 * 
	 * @since  1.0.0
	 * 
	 * @param array $args {
	 *     Associative array of arguments.
	 * 
	 *     @type int $submission_id The ID of the submission to retrieve.
	 *     @type int $form_id       The ID of the form associated with the submission.
	 * }
	 * 
	 * @return array An associative array containing:
	 *               - `name` (string|false): The mapped name value, or `false` if unavailable.
	 *               - `email` (string|false): The mapped email value, or `false` if unavailable.
	 *               - `subject` (string|false): The mapped subject value, or `false` if unavailable.
	 *               - `fields` (array): An array of field labels as keys and their corresponding values as values.
	 * 
	 *               Example return value:
	 *               [
	 *                   'name'    => 'John Doe',
	 *                   'email'   => 'johndoe@foo.bar',
	 *                   'subject' => 'Form Inquiry',
	 *                   'fields'  => [
	 *                       'Name'     => 'John Doe',
	 *                       'Email'    => 'johndoe@foo.bar',
	 *                       'Message'  => 'Hello, I need help...',
	 *                   ],
	 *               ]
	 */
	public function get_submission_data( $args ) {
		$submission_id = $args['submission_id'];
		$form_id = $args['form_id'];
		$result = ['name' => false, 'email' => false, 'subject' => false];
		
    	// Check if Ninja Forms is active
		if ( class_exists('Ninja_Forms') ) {
			// Build an array of field IDs mapped to field labels
			$field_labels = array();
			$fields = \Ninja_Forms()->form( $form_id )->get_fields();
			foreach ( $fields as $field ) {
				$field_id = $field->get_id();
				$field_label = $field->get_setting('label');
				$field_labels[ $field_id ] = $field_label;
			}
			
			// Retrieve the submission object by ID
			$sub = \Ninja_Forms()->form()->get_sub( $submission_id );
			$field_values = $sub->get_field_values();

			// Retrieve custom field mappings for the form
			$map = array();
			$mapping = get_option( 'inbound_organizer_ninjaforms_layouts' );
			if ( $mapping && array_key_exists( $form_id, $mapping ) ) {
				$map = $mapping[ $form_id ];
				$result = ['name' => '-', 'email' => '-', 'subject' => '-'];
			};

			 // Process field values
			foreach ( $field_values as $field_id => $field_value ) {
				/**
				 * Field values have the following structure:
				 * - `_field_x` => "a value"
				 * - `key`      => "a value"
				 * - `_field_y` => "b value"
				 * - `key`      => "b value"
				 * 
				 * This means there are duplicates, so we need to:
				 * 1. Ignore duplicates.
				 * 2. Extract the field number from the `_field_` prefix in the key.
				 */

				// Skip non-field keys and standardize field ID
				if ( strpos( $field_id, '_field_' ) !== false ) {
					//Strip the _field_ string so that a number remains
					$field_id = str_replace('_field_', '', $field_id );
				} else {
					continue;
				}
				  
				// Add the field value to the result if not empty
				$map_key = array_search( $field_id, $map );
				if ( $map_key ) {
					$result[ $map_key ] = $field_value;
				}

				//Only add rows that have a value
				if ( !empty( $field_value ) ) {	
					//Add it to the array
					$result['fields'][ $field_labels [ $field_id ] ] = $field_value;
				}
			}
		}

		return $result;
	}

	/**
     * Save or update the layout configuration for a specific form.
     *
     * This function manages layout settings for forms by storing them in the 
     * `inbound_organizer_ninjaforms_layouts` WordPress option. If the option does not exist, 
     * it initializes it as an empty array before updating the layout for the specified form.
     *
     * @since 1.0.0
     * 
     * @param int   $form The ID of the form for which the layout is being set.
     * @param array $args An associative array of layout settings for the form. Expected keys:
     *                    - `name` (string): The field name for the recipient's name.
     *                    - `email` (string): The field name for the recipient's email.
     *                    - `subject` (string): The field name for the message subject.
     * 
     * @return bool True on successful update of the layout settings.
     */
	public function set_layout( $form_id, $args ) {
		// Retrieve existing layouts from the WordPress options table.
		$mapping = get_option( 'inbound_organizer_ninjaforms_layouts' );
		
    	// If the option doesn't exist or is not an array, initialize it as an empty array.
		if ( !is_array( $mapping ) ) {
			$mapping = array();
		}

		// Update or add the layout entry for the specified form.
		$mapping[ $form_id ] = [ 'name' => $args['name'], 
							  'email' => $args['email'],
							  'subject' => $args['subject'] ];

		// Save the updated layouts back to the WordPress options table.				  
		$result = update_option( 'inbound_organizer_ninjaforms_layouts', $mapping );

		return true;
	}

	/**
     * Retrieve the layout configuration for a specific form.
     *
     * This function fetches the layout settings for a given form ID from the 
     * `inbound_organizer_ninjaforms_layouts` WordPress option. If no layout is found for the form, 
     * it returns a default layout with empty values for `name`, `email`, and `subject`.
     *
     * @since 1.0.0
     * 
     * @param int $form The ID of the form whose layout configuration is being retrieved.
     * 
     * @return array An associative array containing the layout configuration for the form. Keys:
     *               - `name` (string): The field name for the recipient's name.
     *               - `email` (string): The field name for the recipient's email.
     *               - `subject` (string): The field name for the message subject.
     *               If no configuration is found, the default structure is:
     *               [
     *                   'name'    => '',
     *                   'email'   => '',
     *                   'subject' => ''
     *               ]
     */
	public function get_layout( $form_id ) {
		// Default layout structure with empty values.
		$result = ['name' => '', 'email' => '', 'subject' => ''];

		// Retrieve the stored layouts from the WordPress options table.	
		$mapping = get_option( 'inbound_organizer_ninjaforms_layouts' );

		// If layouts exist and the form ID is found, return its layout configuration.
		if ( is_array( $mapping ) && array_key_exists( $form_id, $mapping ) ) {
			$result = $mapping[ $form_id ];
		}

		return $result;
	}

	/**
	 * Retrieve form fields for a specific Ninja Forms Form ID.
	 * 
	 * This function retrieves all fields for the specified form using the Ninja Forms API.
	 * Field data is returned as an associative array, where the keys are the field IDs
	 * and the values are the corresponding field labels.
	 * 
	 * @since 1.0.0
	 * 
	 * @param int $form_id The ID of the Ninja Forms form for which fields are retrieved.
	 * 
	 * @return array An associative array of form fields, where:
	 *               - `key` (int|string): The field ID.
	 *               - `value` (string): The field label.
	 *               
	 *               Example:
	 *               [
	 *                   1 => 'Name',
	 *                   2 => 'Email',
	 *                   3 => 'Message',
	 *               ]
	 *               If no fields are found or Ninja Forms is not active, an empty array is returned.
	 * 
	 */
    public function get_fields_inbound( $form_id ) {
		$result = array();

		// Check if the Ninja Forms plugin is active
		if ( class_exists('Ninja_Forms') ) {
			// Retrieve the fields for the specified form using the Ninja Forms API
			$fields = \Ninja_Forms()->form( $form_id )->get_fields();

			// Iterate through the fields and map field IDs to their labels
			foreach ( $fields as $field ) {
				$field_id = $field->get_id();
				$field_label = $field->get_setting('label');
				$result[ $field_id ] = $field_label;
			}
		}

		return $result;
	}

    /**
	 * Get form definitions of Ninja Forms
	 * Only include those forms that actually have entries.
	 * 
	 * These form definitions are stored as records in the table nf3_forms.
	 *  
	 * @since  1.0.0
	 * @global $wpdb	Instantiation of the wpdb class to connect with the WordPress database.
	 * 
	 * @return   array {
	 * 		@type integer	$id	The form_id, which is the post_id.
	 * 		@type string 	$title The form_title, which is the post_title.
	 * }
	 */
	public function get_forms() {
		global $wpdb;

		//Get all rows from the nf3_forms table
		//A direct database call is the only method to retrieve this data
        $forms = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}nf3_forms", ARRAY_A );
		$result = array();
		foreach ( $forms as $form ) {
            //Check if this form has submissions
			//A direct database call is the only method to retrieve this data			
			$submissions = $wpdb->get_results( 
				$wpdb->prepare( "SELECT * FROM $wpdb->posts INNER JOIN $wpdb->postmeta ON $wpdb->posts.`ID` = $wpdb->postmeta.`post_id` WHERE `meta_key` = '_form_id' AND `meta_value` = %d", 
				$form['id'] ), 
				ARRAY_A );
			
            //Only return the form if there are related form submissions
            if ( !empty( $submissions ) )  {
			    $result[] = array( 'id'    => $form['id'],
							       'title' => $form['title'] );
            }
		}
		return $result;
	}

}