<?php
namespace Inbound_Organizer;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * The core plugin class.
 *
 * This is used to define internationalization and admin-specific hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Inbound_Organizer
 * @subpackage Inbound_Organizer/includes
 * @author     DIM Plugins <robin@internetmanagers.nl>
 */
class Inbound_Organizer {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Inbound_Organizer_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * These are derived from constants if these are defined. If not, defaults are set.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'INBOUND_ORGANIZER_VERSION' ) ) {
			$this->version = INBOUND_ORGANIZER_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		if ( defined( 'INBOUND_ORGANIZER_NAME' ) ) {
			$this->plugin_name = INBOUND_ORGANIZER_NAME;
		} else {
			$this->plugin_name = 'inbound-organizer';
		}	
		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Inbound_Organizer_Loader. Orchestrates the hooks of the plugin.
	 * - Inbound_Organizer_i18n. Defines internationalization functionality.
	 * - Inbound_Organizer_Admin. Defines all hooks for the admin area.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-inbound-organizer-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-inbound-organizer-i18n.php';

		/**
		 * The classes responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-inbound-organizer-admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-inbound-organizer-plugins.php';

		$this->loader = new Inbound_Organizer_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Inbound_Organizer_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 * 
	 * Changed hook from plugins_loaded to init.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {
		$plugin_i18n = new Inbound_Organizer_i18n();
		$this->loader->add_action( 'init', $plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugins = new Inbound_Organizer_Plugins( $this->get_plugin_name(), $this->get_version() );
		$admin = new Inbound_Organizer_Admin( $this->get_plugin_name(), $this->get_version(), $plugins );
		
		// Check which form plugins are active and add hooks and filters accordingly to trigger additions to the board.
		// Changed hook from init to plugins_loaded to prevent calling is_plugin_active() too soon.
		$this->loader->add_action( 'plugins_loaded', $plugins, 'check_plugins');

		//Register a custom post type to keep track of form submissions
		$this->loader->add_action( 'init', $admin, 'register_post_type' );
		
		// Add script and style files
		$this->loader->add_action( 'admin_enqueue_scripts', $admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $admin, 'enqueue_scripts' );
		
		// Add menu to WordPress
		$this->loader->add_action( 'admin_menu', $admin, 'add_menu');
		
		// Add AJAX-action to drag & drop a lead to another column (update tracking status)
		$this->loader->add_action( 'wp_ajax_inbound-organizer_update_message_tracking_status', $admin, 'update_message_tracking_status');
	
		// Add AJAX-action to render a lead in detail
		$this->loader->add_action( 'wp_ajax_inbound-organizer_render_message_details', $admin, 'render_message_details');
	
		// Add AJAX-action to trash a lead
		$this->loader->add_action( 'wp_ajax_inbound-organizer_trash_message', $admin, 'trash_message');

		// Add AJAX-action to save notes for a lead
		$this->loader->add_action( 'wp_ajax_inbound-organizer_save_message_notes', $admin, 'save_message_notes');
		
		// Add AJAX-action to save the card layout for an unknown form type
		$this->loader->add_action( 'wp_ajax_inbound-organizer_save_layout', $admin, 'save_layout');

		// Add AJAX-action to load more messages (pagination)
		$this->loader->add_action( 'wp_ajax_inbound-organizer_load_more_messages', $admin, 'load_more_messages');

		// Add AJAX-action to refresh messages (after message movement)
		$this->loader->add_action( 'wp_ajax_inbound-organizer_refresh_messages', $admin, 'refresh_messages');

		// Add AJAX-action to update card color
		$this->loader->add_action( 'wp_ajax_inbound-organizer_update_message_color', $admin, 'update_message_color');

		// Add AJAX-action to add messages to Inbound Organizer received prior to plugin install
		$this->loader->add_action( 'wp_ajax_inbound-organizer_add_messages', $admin, 'add_messages');

		// Initialize settings page
		$this->loader->add_action( 'admin_init', $admin, 'init_settings');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Inbound_Organizer_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}
}
